#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
uGao pacakge
Calculator for the DetectorViewer

---- Original version
Emaki package
Emaki calculator
Author: ITO, Takayoshi (CROSS)
Last update: Feb. 4th, 2013
"""
__author__ = "ITO, Takayoshi (CROSS)"
__version__ = "19.03.10b"
__date__ = "25th Apr. 2019"

import numpy as np

########################################
def PutGauss( args ):
    A = args[0]
    cx = args[1]
    w = args[2]
    x = args[3]
    return A*np.exp(-(x - cx)**2/(2.0*w**2))

########################################
def PutResultOfUserDefinedFormula( args ):
    x, y, z = args[0]
    formula = args[1]
    try:
        return eval(formula)
    except NameError:
        return eval('np.'+formula)

########################################
def PutTwoTheta(coordinate):
    """
    実験室座標系の位置座標から散乱角を返す。
    ビーム軸はz軸。
    @param coordinate (list) 実験室座標
    @retval (float) 散乱角。単位は度。
    """
    return np.degrees(np.arccos(coordinate[2]/np.sqrt(np.sum(coordinate**2))))

########################################
def RotateWXYZ(params, th=None, v=None):
    """
    pをv軸回転で、th度回転。
    @param p (list) 座標を表すlist
    @param th (float) 回転角度
    @param v (list) 回転軸を表すlist
    @retval (numpy.array) 回転した結果を表すnumpy.array
    """
    if th is None and v is None:
        p, th, v = params[:]
    else:
        p = params[:]
    p_np = np.array(p)
    v_np = np.array(v)
    a_np = v_np/np.sqrt(np.sum((v_np**2))) # a_np: vの単位ベクトル。
    th_rad = np.radians(th)
    return np.cos(th_rad)*p_np + (1-np.cos(th_rad))*np.dot(p_np, a_np)*a_np+np.sin(th_rad)*np.cross(a_np, p_np)

########################################
def PutAngleBetweenTwoVectors( v0, v1):
    """
    二つのベクトルがなす角を返す．単位は度．
    @param v0 (numpy.array) 角度の基準となるベクトル。
    @param v1 (numpy.array) 調べたいベクトル。
    @retval (float) 角度。単位は度。
    """
    val = np.dot( v0, v1)/ (np.linalg.norm(v0)* np.linalg.norm(v1))
    ## 浮動小数点につきまとう誤差のために絶対値が1.0を越える場合がある．
    ## 小数点8桁までとると、0.000015 rad以内の精度になるので1.0との差が1.0e-10よりも小さい場合は1.0に丸める．
    if np.abs(val) > 1.0:
        if (np.abs(val) - 1.0) < 1.0e-10:
            val = np.rint(val)
        else:
            raise UserWarning('Cannot calculate the angle.')
    phi = np.arccos( val )
    return np.degrees(phi)

########################################
def PolarToCartesian( r, th, phi):
    """
    極座標を直交座標にして返す。単位は度。
    @param r (float) 動径
    @param th (float) 極角
    @param phi (float) 方位角
    @retval (float, float, float) 極座標
    """
    x = r*np.sin(np.radians(th))*np.cos(np.radians(phi))
    y = r*np.sin(np.radians(th))*np.sin(np.radians(phi))
    z = r*np.cos(np.radians(th))
    return x, y, z

########################################
def CartesianToPolar( x, y, z):
    """
    直交座標を極座標にして返す。単位は度。
    @param x (float) X座標
    @param y (float) Y座標
    @param z (float) Z座標
    @retval (float, float, float) 直交座標
    """
    r = np.sqrt(x**2+y**2+z**2)
    th = np.degrees(np.arccos(z/r))
    phi = np.degrees(((1, -1)[int(np.signbit(y))])*np.arccos(x/np.sqrt(x**2+y**2)))
    return r, th, phi




#!/usr/bin/python
# -*- coding: utf-8 -*-

import os,sys,time
ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree,fromstring,XML,tostring
else:
    from xml.etree.ElementTree import ElementTree,fromstring,XML,tostring

from numpy import *
import time

import Manyo as mm
import Manyo.Utsusemi as mu
import vis.VisContMParams as VCP
      
################################
#      CuiCtrlVisContM
################################
class CuiCtrlVisContM(object):
    """
    Class to control VisualContM from CUI
    
    ver 0.1 [110726] first release
    Change History
    [inamura 160619] Bugfix of treating SampleRotateSteps
    """
    Def_type = ["Visual","OutToD4Mat"]
    #####################################
    def __init__(self, filepath="", type="OutToD4Mat", ecm=None):
        """
        Constructor
        
        @param self
        @param filepath (String) path to xml file of parameters
        @param ecm (ElementContainerMatrix)
        @retval None
        """
        if not type in self.Def_type:
            print "type=",type," is invalid."
            raise
        
        self.VCP = VCP.VisContMParams()

        self.type     = type
        self.viewAxis = None
        self.param    = None
        self.plotid   = None
        self.ecm      = ecm
        self.map      = None
        self.folding  = [0,0,0,0]
        self.VisualCalcSqe = None
        self.MessageTag = "CuiCtrlVisContM >>>"
        
        if filepath!="":
            try:
                self.VCP.SetParamFile( filepath )
            except:
                print "File not found.",filepath
                raise
            try:
                self.VCP.LoadVisualContParams()
            except:
                print "XML format may be invalid."
                raise

            self.MakeSliceParams()

        if self.ecm!=None:
            self.SetECM()

        
    #####################################
    def MakeSliceParams(self,type=""):
        """
        Make parameters for slicing from imported XML.
        @param type (String) the name of visualization software to be used 
        """
        
        if (self.VCP.proj_axes == None) or (self.VCP.paxis_info == None):
            return

        if (type!="") and (type in self.Def_type):
            self.type = type

        """
        if self.VCP.EnableVisType!=self.type:
            ss="The type of parameter XML is not correct for "+self.type
            raise "IncorrectXMLFile",ss
        """
            
        # make parameter for slicing funcions on VisualCont(2)
        self.viewAxis = zeros([4,4], float64)
        self.param = zeros([12],float64)

        self.plotid = [ [], [], [] ]
        
        if self.type=="Visual":
            print self.MessageTag,"Type=Visual"

            shift = 0
            index = 0
            flag_t = 0

            for i in range(4):
                (type_str,range_axis,folding) = self.VCP.paxis_info[ str(i) ]

                va = self.VCP.proj_axes[ str(i) ]
                for j in range(4):
                    self.viewAxis[index][j]=va[j]
                if type_str=="x":
                    self.plotid[0].append(index)
                    for j in range(len(range_axis)):
                        self.param[j]=range_axis[j]
                   
                elif type_str=="y":
                    self.plotid[1].append(index)
                    for j in range(len(range_axis)):
                        self.param[j+3]=range_axis[j]
                    
                elif (type_str=="t"):
                    self.plotid[2].append(index)
                    if flag_t==0:
                        for j in range(len(range_axis)):
                            self.param[j+6]=range_axis[j]
                        flag_t=1
                    else:
                        for j in range(len(range_axis)):
                            self.param[j+8]=range_axis[j]

                else:
                    print self.MessageTag,"Unknown type = ",type_str
                    raise

                self.folding[index]=folding

                index+=1


        elif self.type=="OutToD4Mat":
            print self.MessageTag,"Type=OutToD4Mat"

            for i in range(4):
                (type_str,range_axis,folding) = self.VCP.paxis_info[ str(i) ]

                va = self.VCP.proj_axes[ str(i) ]
                for j in range(4):
                    self.viewAxis[i][j]=va[j]

                if (type_str=="t"):
                    if (len(range_axis)==2):
                        range_axis.append(range_axis[1]-range_axis[0])
                    elif (len(range_axis)==3):
                        range_axis[2] = range_axis[1]-range_axis[0]
                    else:
                        print self.MessageTag,"index = %d : range_axis number(%d) is invalid" % (i,len(range_axis))
                        raise

                for j in range(len(range_axis)):
                    self.param[i*3 + j]=range_axis[j]

                self.folding[i]=folding
                
        else:
            print self.MessageTag,"Type is invalid.",type

    #####################################            
    def SetPhi(self, phi=0.0):
        """
        Set Phi ( fai in KCS products )
        @param phi (Double) phi angle from given orientation of sample
        """
        if self.VCP.Header=={}:
            print self.MessageTag,"Original XML file is not loaded."
            raise
        
        #self.VCP.smpInfo.fai = float(fai)
        self.VCP.SetOrientationPhi(phi)

    #####################################
    def SetECM(self, ecm=None):
        """
        Set ElementContainerMatrix for slicing.
        @param ecm (ElementContainerMatrix)
        """
        
        #if ecm!=None and type(ecm)==type(mm.ElementContainerMatrix()):
        if ecm!=None and isinstance( ecm, mm.ElementContainerMatrix ):
            self.ecm = ecm

    #####################################
    def DoProjection(self):
        """
        """
        if self.VCP.Header=={}:
            print self.MessageTag,"Did not make parameters."
            raise

        print self.MessageTag,"Projection..."

        ## Get parameters from XML
        LC  = self.VCP.Header['SimpleLatticeConsts']
        UV  = self.VCP.Header['SampleUvec']
        VV  = self.VCP.Header['SampleVvec']
        if 'SampleRotatePhi' in self.VCP.Header:
            PHI = self.VCP.Header['SampleRotatePhi']
        else:
            PHI = 0
        ##[inamura 160619]-->
        if  self.VCP.Header.has_key('SampleRotateSteps'):
            RL= self.VCP.Header['SampleRotateSteps']
        else:
            RL=None
        ## Set parameters to Header of ElementContianerMatrix
        ecm_h = self.ecm.PutHeaderPointer()
        lcv = mm.MakeDoubleVector()
        for i in range(6):
            lcv.append( LC[i] )
        if ecm_h.CheckKey("SimpleLatticeConsts")==1:
            ecm_h.OverWrite("SimpleLatticeConsts",lcv)
        else:
            ecm_h.Add("SimpleLatticeConsts",lcv)
            
        uv = mm.MakeDoubleVector()
        vv = mm.MakeDoubleVector()
        for i in range(3):
            uv.append( UV[i] )
            vv.append( VV[i] )
        if ecm_h.CheckKey("SampleUvec")==1:
            ecm_h.OverWrite("SampleUvec",uv)
        else:
            ecm_h.Add("SampleUvec",uv)
        if ecm_h.CheckKey("SampleVvec")==1:
            ecm_h.OverWrite("SampleVvec",vv)
        else:
            ecm_h.Add("SampleVvec",vv)
        
        if ecm_h.CheckKey("SampleRotatePhi")==1:
            ecm_h.OverWrite("SampleRotatePhi", PHI )
        else:
            ecm_h.Add("SampleRotatePhi", PHI )
        
        ##[inamura 160619]-->
        ##[inamura 150907]-->
        sample_rot_steps = mm.MakeDoubleVector()
        if RL!=None and len(RL)!=0:
            for a_rl in RL:
                if a_rl[0]=="X":
                    sample_rot_steps.append( 0.0 )
                elif a_rl[0]=="Y":
                    sample_rot_steps.append( 1.0 )
                elif a_rl[0]=="Z":
                    sample_rot_steps.append( 2.0 )
                else:
                    continue
                sample_rot_steps.append( a_rl[1] )
        else:
            sample_rot_steps.append( 1.0 ) ## Axis-Y is 1
            sample_rot_steps.append( PHI ) ## rotate angle

        ##<--[inamura 160619]
        if ecm_h.CheckKey("SampleRotateSteps")==1:
            ecm_h.OverWrite("SampleRotateSteps", sample_rot_steps )
        else:
            ecm_h.Add("SampleRotateSteps", sample_rot_steps )
        ##<--[inamura 150907]
        
        ## Calc projection
        self.VisualCalcSqe = mu.UtsusemiSqeCalc()
        
        self.VisualCalcSqe.SetTarget( self.ecm )
        viewAxis_in = mm.MakeDoubleVector()
        for i in range(4):
            for j in range(4):
                viewAxis_in.append( self.viewAxis[i][j] )
        all_u = self.VisualCalcSqe.Projection(viewAxis_in)
        
        
    #####################################
    def DoSlice(self):
        """
        Make sliced data from given ElementContainerMatrix and slicing parameters.
        
        """
        
        self.DoProjection()
            
        ## Slicing
        Ranges = []
        AxType = mm.MakeStringVector()
        Foldings = mm.MakeDoubleVector()
        for i in range(4):
            (type_str, range_axis, folding) = self.VCP.paxis_info[ str(i) ]
            print "type_str=",type_str
            AxType.append( type_str )
            Foldings.append( folding )
            Ranges.append( mm.MakeDoubleVector() )
            for j in range( len(range_axis) ):
                Ranges[i].append( range_axis[j] )

        ret = self.VisualCalcSqe.Slice( Ranges[0],Ranges[1],Ranges[2],Ranges[3],
                                        AxType,Foldings )
        ## Make map format                
        xv = self.VisualCalcSqe.PutXArray()
        yv = self.VisualCalcSqe.PutYArray()
        II = self.VisualCalcSqe.PutDArray()
        EE = self.VisualCalcSqe.PutEArray()
        print "size=",xv.size(),yv.size(),II.size(),EE.size()
        h = []
        v = []
        for i in range(xv.size()):
            h.append(xv[i])
        for i in range(yv.size()):
            v.append(yv[i])
        
        h = array(h)
        v = array(v)

        H, V = meshgrid(h, v)

        numx = xv.size()-1
        numy = yv.size()-1
        print "numx,numy=",numx,numy
        data =zeros((numy,numx), float64)
        err =zeros((numy,numx), float64)
        
        for j in range(numx):
            for k in range(numy):
                data[k,j] = II[ j*numy + k ]
                err[k,j] = EE[ j*numy + k ]

        self.map = ( data, err, H, V, 1 )

    #####################################
    def DoSliceToEca(self):
        """
        Make sliced data from given ElementContainerMatrix and slicing parameters.
        
        """
        
        self.DoProjection()
            
        ## Slicing
        Ranges = []
        AxType = mm.MakeStringVector()
        Foldings = mm.MakeDoubleVector()
        for i in range(4):
            (type_str, range_axis, folding) = self.VCP.paxis_info[ str(i) ]
            print "type_str=",type_str
            AxType.append( type_str )
            Foldings.append( float(folding) )
            Ranges.append( mm.MakeDoubleVector() )
            for j in range( len(range_axis) ):
                Ranges[i].append( range_axis[j] )

        ret = self.VisualCalcSqe.Slice( Ranges[0],Ranges[1],Ranges[2],Ranges[3],AxType,Foldings,True )
        ECA = mm.ElementContainerArray()
        self.VisualCalcSqe.SetSlicedElementContainerArray( ECA )
        return ECA
                
    #####################################    
    def SendToM2Plot(self):
        """
        Send sliced data to M2Plot.
        """
        if self.map == None:
            print self.MessageTag,"Did not do slicing."
            raise
        
        import vis.M2Plot as M2P
        return M2P.M2Plot(self.map)

    #####################################
    def SaveFigureOfM2Plot(self,filepath="test.png",zmin=-1,zmax=-1,isLogPlot=False):
        """
        Save a figure for sliced data plotted by CuiD2Chart
        [inamura 120426]
        @param filepath (string) Output file (extension must be .png)
        @param zmin (double) min z axis range
        @param zmax (double) max z axis range
        @param isLogPlot (bool) use log scaling on Intensity or not
        @retval None
        """
        if self.map == None:
            print self.MessageTag,"Did not do slicing."
            raise


        #make Main Title
        Titles=["","","",""]
        Ei = self.ecm.PutHeader().PutDouble("Ei")
        sample = self.VCP.Header['SampleName']
        Titles[0] = "%s ( Ei= %.1f (meV) )" % ( sample, Ei )
        
        # make Sub Title
        UV = self.VCP.Header['SampleUvec']
        VV = self.VCP.Header['SampleVvec']
        str1 = "U=[%4.2f, %4.2f, %4.2f]\n" % (UV[0],UV[1],UV[2])
        str2 = "V=[%4.2f, %4.2f, %4.2f]\n" % (VV[0],VV[1],VV[2])
        if 'SampleRotatePhi' in self.VCP.Header:
            str3 = "Psi=%5.2f\n" % self.VCP.Header['SampleRotatePhi']
        else:
            rotstep_list = self.VCP.Header['SampleRotateSteps']
            tmp_rot_str="RotStep="
            for a_rotstep in rotstep_list:
                tmp_rot_str+=("%s:%g,"%(a_rotstep[0],a_rotstep[1]))
            str3 = "%s\n"%(tmp_rot_str[:-1])

        AX1=self.VCP.proj_axes["0"]
        AX2=self.VCP.proj_axes["1"]
        AX3=self.VCP.proj_axes["2"]
        AX4=self.VCP.proj_axes["3"]
        str4 = "ax1=[%4.2f,%4.2f,%4.2f,%4.2f]\n" % (AX1[0],AX1[1],AX1[2],AX1[3])
        str5 = "ax2=[%4.2f,%4.2f,%4.2f,%4.2f]\n" % (AX2[0],AX2[1],AX2[2],AX2[3])
        str6 = "ax3=[%4.2f,%4.2f,%4.2f,%4.2f]\n" % (AX3[0],AX3[1],AX3[2],AX3[3])
        str7 = "ax4=[%4.2f,%4.2f,%4.2f,%4.2f]\n" % (AX4[0],AX4[1],AX4[2],AX4[3])

        nn = []
        str8 = ""
        for i in range(len(self.VCP.paxis_info)):
            PX=self.VCP.paxis_info[str(i)]
            if PX[0]=="t":
                str8 += "Thick=[ax%d:%s - %s]\n" % ( i,PX[1][0],PX[1][1] )
            if PX[0]=="x":
                Titles[2]=self.VCP.proj_axes[str(i)][4]
            if PX[0]=="y":
                Titles[3]=self.VCP.proj_axes[str(i)][4]
                
        str8 = str8[:-1]
        
        Titles[1] = str1+str2+str3+";"+str4+str5+str6+str7+";"+str8

        # make figure
        import vis.CuiD2Chart as CDC

        cdc = CDC.CuiD2Chart( self.map, zmin, zmax, Titles )
        cdc.MakePlot()
        cdc.SaveFile( filepath )

        del cdc
        
    #####################################
    def OutToD4Mat(self, filename):
        """
        """
        filepath = str(filename)
        
        self.DoProjection()
        Ranges = []
        #Foldings = mm.MakeInt4Vector()
        Foldings = mm.MakeDoubleVector()
        for i in range(4):
            (type_str, range_axis, folding) = self.VCP.paxis_info[ str(i) ]
            Foldings.append( folding )
            Ranges.append( mm.MakeDoubleVector() )
            for j in range( len(range_axis) ):
                Ranges[i].append( range_axis[j] )
        
        ret = self.VisualCalcSqe.ConvertToD4Mat( filepath,
                                                 Ranges[0],Ranges[1],Ranges[2],Ranges[3],
                                                 Foldings )



###########################    
def Test(filepath,ecm,type="VisualCont"):
    cui = CuiCtrlVisCont(filepath,type,ecm)
    cui.DoSlice()
    cui.SendToM2Plot()
    
    return cui



#! /usr/bin/env python3
from __future__ import print_function
import os
import sys
import DR
import Cmm
import utsusemi.ana.Reduction.BaseCommandsInEla as BC

#####################
# modify here
####################
Ei = 30.04
delta_hw = 0.2
hw_min = -5.0
hw_max = 28.0
maskfile = "default"
normFactor = 1000000.0
XtalParam = "XtalParam.xml"
savePathDir = "./data"
gonioValueAtZero = 31.032
#########################

argv = sys.argv
print("len(argv)={}".format(len(argv)))

listFileName = ""
if len(argv) == 2:
    if os.path.exists(argv[1]):
        listFileName = argv[1]
        print("RunList=[]".format(listFileName))
    else:
        raise UserWarning("Not found file=%s" % (argv[1]))
else:
    print("Usage::")
    print("        $ ./{} <filename of runlist>".format(argv[0]))
    print("        ")
    print("        $ python {} <filename of runlist>".format(argv[0]))
    sys.exit(0)

if not os.path.exists(savePathDir):
    print(" savePathDir = {} is not found. Making new ".format(savePathDir))
    try:
        os.makedirs(savePathDir)
    except:
        raise UserWarning("Cannot make new folder(" + savePathDir + ")")

paramDic = {"normFactor": normFactor, "gonio": gonioValueAtZero}
conds = BC.LoadRunListToExtract(listFileName, paramDic)

for (ScanNo, PhiValue, Norm) in conds:
    # Cmm.CopyData(ScanNo)
    print("==============[ %06d,%8.3f,%10.5f ]" % (ScanNo, PhiValue, Norm))
    savePath = os.path.join(savePathDir, ("S%06d.vbin" % (ScanNo)))
    DAT = DR.GetDataOfMonochroEi2(ScanNo, Ei, delta_hw, hw_min, hw_max, maskfile, Norm)
    Cmm.VisContMOutputFileByPhi(DAT, XtalParam, PhiValue, savePath)
    del DAT



# -*- coding: utf-8 -*-

import Manyo as m
import Manyo.MLF as mlf


def DumpToFile( em, FileName ):

    fout = open( FileName, "w" )
    x = em( em.PutXKey() )
    y = em( em.PutYKey() )
    e = em( em.PutEKey() )
    
    if x.size() == y.size()+1:
        # if histogram type data
        for i in range( y.size() ):

            s = str( x[i] )   + " " + str( 0.0 ) + "\n"
            fout.write( s )
            
            s = str( x[i] )   + " " + str( y[i] ) + "\n"
            fout.write( s )
            s = str( x[i+1] ) + " " + str( y[i] ) + "\n"
            fout.write( s )
        s = str( x[ x.size()-1 ] )   + " " + str( 0.0 ) + "\n"
        fout.write( s )
            
    else:
        # if disperse type data
        s = str( x[i] )   + " " + str( y[i] ) + "\n"
        fout.write( s )
        
    fout.close()

def DumpToFileErr( em, FileName ):
    
    fout = open( FileName, "w" )
    x = em( em.PutXKey() )
    y = em( em.PutYKey() )
    e = em( em.PutEKey() )
    
    if x.size() == y.size()+1:
        # if histogram type data
        for i in range( y.size() ):
            s = str( ( x[i] + x[i+1] )/2.0 ) + " " + \
                str( y[i] ) + " " + \
                str( e[i] ) + "\n"
            fout.write( s )
            
    else:
        # if disperse type data
        s = str( x[i] ) + " " + \
            str( y[i] ) + " " + \
            str( e[i] ) + "\n"
        fout.write( s )
        
    fout.close()

    



from __future__ import print_function

import Manyo
import Manyo.MLF as mm
import Manyo.Utsusemi as mu
import Manyo.SAS as ms
import utsusemi.ana.Reduction.BaseCommands as BC
import utsusemi.SAS.ana.Reduction.CorrBaseCommands as CBC
from utsusemi.SAS.ana.Reduction.SASUtils import SASBankDic

import os
import glob

"""
Base Functions for SAS
Hist.py
- GetHistogramSAS
- GetMonHistSAS
- GetPulseHeightSAS
- GetMonHistByPhSAS
- MergeSMSC4DetMap

"""
########################################
def CheckT0bUpdate(runNo):
    """
    """
    # <-- TI [20140403]
    sys_name = mu.UtsusemiEnvGetInstCode()
    edb_base_dirs = []
    path_tmpl = os.path.join(mu.UtsusemiEnvGetInstDataDir(), sys_name+"%0d")
    for i in range(0,100):
        if os.path.exists( path_tmpl%(i) ):
            edb_base_dirs.append( path_tmpl%(i) )
    #edb_base_dirs = [os.path.join(os.environ['UTSUSEMI_DATA_DIR'], sys_name),
    #                 os.path.join(os.environ['UTSUSEMI_DATA_DIR'], sys_name+"00")]
    t0b_dir = os.path.join(mu.UtsusemiEnvGetUserDir(), 'ana', 'tmp')
    t0b_template = '%s%06d_??_???_???.t0b'%(sys_name, runNo)
    t0b_list = glob.glob(os.path.join(t0b_dir, t0b_template))
    for edb_base_dir in edb_base_dirs:
        edb_dir_candidates = glob.glob(os.path.join(edb_base_dir, '%s%06d_????????'%(sys_name, runNo)))
        if len(edb_dir_candidates) == 0:
            #raise UserWarning, "edb not fount"  ## [20150311]YI -->
            pass
        else:
            edb_dir_candidates.sort()
            edb_dir = edb_dir_candidates[-1]
            for t0b in t0b_list:
                edb_template = os.path.join(edb_dir, '%s_???.edb'%(os.path.basename(t0b)[:-8]))
                edb_list = glob.glob(edb_template)
                if len(edb_list)==0:
                    continue
                edb_list.sort()
                edb = edb_list[-1]
                t0b_stat = os.stat(t0b)
                edb_stat = os.stat(edb)
                if t0b_stat.st_mtime <= edb_stat.st_mtime:
                    os.system("rm %s"%t0b)
    ## <-- [20150311]YI
    # --> TI [20140403]

########################################
def GetHistogramBaseSAS( runNo="0", Xrange="0.7 7.6 0.1 dL", Bank="SM", TimeSlice="-1 -1", frameBoundary="0.0", isTimeFoc=False, SolAngCor=True, paramFiles="- -", isPH=False, PHrange="0 4096" ):
    """
    Base Function to Load EventData to produce histogram for SAS
    GetHistogramBaseSAS( runNo=0, Xrange="0.7 7.6 0.1 dL", Bank="SM", TimeSlice="-1 -1", frameBoundary="0.0", isTimeFoc=False, SolAngCor=True, paramFiles="- -", isPH=False, PHrange="0 4096" )

    @param runNo     (string)     Run Number
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof, d, rd, ene]
    @param Bank      (string)  "SM"...  Monitors by "MON1" or "MON2"
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (string) "frameBoundary" or "frameBoundary TofMask_min TofMask_max"
    @param isTimeFoc (bool)    True or False
    @param SolAngCor  (bool)    True or False for Solid Angle Correction
    @param paramFiles (string) "WiringInfo DetectorInfo"
    @param isPH (bool)
    @param PHrange (string)
    @retval DAT (ElementContainerMatrix)
    """

    ## Check Bank
    detType=""
    mode_no=0 #"PSD"
    #RPMT_PARAM=[128, 1024, 128, 1024, 32] #[<LLD_X>,<HLD_X>,<LLD_Y>,<HLD_Y>,<LmtCoi>]
    RPMT_LLDHLD = [0, 0, 0, 0] #[<LLD_X>,<HLD_X>,<LLD_Y>,<HLD_Y>]
    RPMT_COI = 32 #<LmtCoi>
    if Bank == "RPMTo":
        detType="RPMTo"
        mode_no=3
    elif Bank == "MON1":
        detType="N2MON1"
        mode_no=1
    elif Bank == "MON2":
        detType="N2MON2"
        mode_no=2
    elif Bank == "MON3":
        detType="N2MON3"
        mode_no=4
    elif Bank.find("RPMT")==0: #RPMT,<LLD_X>,<HLD_X>,<LLD_Y>,<HLD_Y>,<LmtCoi>
        detType="PSD"
        mode_no=5
        rpmt_params=Bank.split(",")
        if len(rpmt_params)>1:
            RPMT_LLDHLD[0] = int(rpmt_params[1])
        if len(rpmt_params)>2:
            RPMT_LLDHLD[1] = int(rpmt_params[2])
        if len(rpmt_params)>3:
            RPMT_LLDHLD[2] = int(rpmt_params[3])
        if len(rpmt_params)>4:
            RPMT_LLDHLD[3] = int(rpmt_params[4])
        if len(rpmt_params)>5:
            RPMT_LLDHLD[4] = int(rpmt_params[5])
        if len(rpmt_params)>6:
            RPMT_COI = int(rpmt_params[6])
    elif Bank in list(SASBankDic.keys()):
        detType="PSD"
        mode_no=0
    elif Bank.upper().strip()=="ALL":
        detType="PSD"
        mode_no=0
    else:
        banks = Bank.split(" ")
        tmpBank = ""
        for a_bank in banks:
            if a_bank!="" or a_bank!=" ":
                num_of_pixels = ""
                ind_deli = a_bank.find(":")
                if ind_deli>0:
                    num_of_pixels = a_bank[ind_deli:]
                    a_bank=a_bank[:ind_deli]
                if a_bank=="SM":
                    tmpBank += "SMBL14"+num_of_pixels+","+"SMBL16"+num_of_pixels+","
                elif a_bank in SASBankDic:
                    tmpBank += (a_bank+num_of_pixels+",")
                else:
                    pass
        if tmpBank!="":
            Bank = tmpBank[:-1]
            detType="PSD"
            mode_no=0
        else:
            raise UserWarning("Bank info is invalid. ["+Bank+"]")

    if isPH:
        if PHrange!="":
            ph_range_list = PHrange.split(" ")
            detType+="-PH[%d:%d]"%(int(ph_range_list[0]),int(ph_range_list[1]))
        else:
            detType+="-PH"

    if Bank=="":
        raise UserWarning("Bank is invalid.")
    else:
        print("========[ Bank = %s ]"%(Bank))
    ## If paramFiles is given.(Not Work, under construction)
    wfile_found_path=""
    dfile_found_path=""
    cfile_found_path=""
    if paramFiles!="- -":
        param_files_list = paramFiles.split(" ")
        if len(param_files_list)>=2:
            if param_files_list[0]!="-":
                wfile_found_path = mu.FindParamFilePath( param_files_list[0] )
                if wfile_found_path=="":
                    raise UserWarning("Given WiringInfo is not found.(%s)"%(param_files_list[0]))
            if param_files_list[1]!="-":
                dfile_found_path = mu.FindParamFilePath( param_files_list[1] )
                if dfile_found_path=="":
                    raise UserWarning("Given DetectorInfo is not found.(%s)"%(param_files_list[1]))
        if len(param_files_list)>=3:
            cfile_found_path = mu.FindParamFilePath( param_files_list[2] )
            if cfile_found_path=="":
                raise UserWarning("Given CaseInfo is not found.(%s)"%(param_files_list[2]))

    ## Check Time Slice
    TimeSliceList=TimeSlice.split(" ")
    time_range=[]
    for ts in TimeSliceList:
        if ts!="":
            time_range.append( float(ts) )

    ## Check Frame Boundary
    frame_bound_list = frameBoundary.split(" ")
    if (len(frame_bound_list)!=1) and (len(frame_bound_list)!=3):
        raise UserWarning("Frame Boundary argument is invalid.")
    frameBoundary_val = ""
    if (frame_bound_list[0]!="0.0"):
        ind = frame_bound_list[0].find(":")
        if (ind<0):
            frameBoundary_val = "2,%s"%frame_bound_list[0] # TOF boundary
        else:
            frameBoundary_val = frame_bound_list[0]        # Lambda or Energy boundary

    #if (frameBoundary_val!=0.0):
    #    mpf.EDW.set_FrameInfo(frameNo=2, boundary=frameBoundary_val )

    ## set tof mask region
    mask_region = ""
    if (len(frame_bound_list)==3):
        mask_region = "%f-%f"%(float(frame_bound_list[1]),float(frame_bound_list[2]))

    ## Make RunNo List
    if type(runNo)==str:
        runNo_list = runNo.split(" ")
        if len(runNo_list)==0:
            raise UserWarning("runNos are invalid")
    elif type(runNo)==int:
        runNo_list = [ str(runNo) ]

    ## Check T0b
    runNo = ""
    for a_runNo in runNo_list:
        CheckT0bUpdate(int(a_runNo))
        runNo += a_runNo+","
        if mode_no==5:
            DaqId = 13 #Default
            ind = Bank.find(":")
            if ind>0:
                DaqId=int(Bank[ind+1:])
            #ConvertFromRPMTK2Psd( int(a_runNo), DaqId, 0, "SAS03")
            ConvertFromRPMTK2Psd(int(a_runNo), DaqId, 0, "SAS03", mode_no, RPMT_LLDHLD, RPMT_COI, dfile_found_path)
    runNo = runNo[:-1]

    ## Make Parameter File to Create Histogram
    GNH = mu.UtsusemiGetNeunetHistogram()

    if GNH.SetRunNo( runNo, wfile_found_path, dfile_found_path, str(mode_no) ):
        pass
    else:
        raise UserWarning("")

    if cfile_found_path!="":
        if GNH.SetCaseInfo(cfile_found_path):
            pass
        else:
            raise UserWarning("CaseInfo is not invalid.")

    conv_params=""
    p_list=Xrange.split(" ")
    if len(p_list)!=4:
        raise UserWarning("param error")
    AxType=p_list[3].lower()
    if AxType=="dl":
        conv_params="lambda,%g,%g,%g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="dl/l":
        conv_params="rlambda,%g,%g,%g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="tof":
        conv_params="tof,%g,%g,%g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="d":
        conv_params="d,%g,%g,%g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType == "rd":
        conv_params="rd,%g,%g,%g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    elif AxType=="ene":
        conv_params="energy,%g,%g,%g"%(float(p_list[0]),float(p_list[1]),float(p_list[2]))
    else:
        raise UserWarning("param error about conv type ({})".format(AxType))

    #print( "Conv Params="+conv_params )
    #print( "DetType="+detType )
    #print( "Bank="+Bank )

    if not GNH.SetConversionParameter( conv_params ):
        raise UserWarning("Conv_params is invalid."+conv_params)

    if not GNH.SetDetParam( detType ):
        if detType!="RPMTo":
            raise UserWarning("DetType is invalid."+detType)

    if mode_no==0:
        GNH.SetDetRange( Bank )

    ## Because T0-ID of NEUNNET is different with monitor(GateNET) or RPMT.
    if mode_no==0:
        GNH.SetFlagOfPulseIdCheck(True)
    else:
        GNH.SetFlagOfPulseIdCheck(False)

    GNH.SetFlagToAddTofBin(True)
    #if frameBoundary_val!=0.0:
    #    GNH.SetFrameInfo( "2,%g"%(frameBoundary_val) )
    if frameBoundary_val!="":
        GNH.SetFrameInfo( frameBoundary_val )
        print("python >> ------------ NEW SetFrameInfo")

    if mask_region!="":
        GNH.SetMaskInfo("NoFile,"+mask_region)

    ## Create Histograms
    data_base_dir = mu.UtsusemiEnvGetDataDir()
    if detType!="RPMTo": ## Neunet series
        print("---- Histogram Create ----")
        dat_list = []
        if len(time_range)==2:
            GNH.SetTimeRange("%g,%g"%(time_range[0],time_range[1]))

        #the path of converted data for RPMT is different from that of original data,
        if mode_no==5:
            mu.UtsusemiClearRunDataPathDict()
        if not GNH.LoadEventDataFiles( data_base_dir, "" ):
            msg = " LoadEventDataFiles failed."
            raise UserWarning(msg)
        v_cases = GNH.PutListOfCases()
        # print("#[inamura 220310] cases.size()={}".format(v_cases.size()))
        caselist = []
        if v_cases.size()==0:
            caselist = [1]
        else:
            for i in range(v_cases.size()):
                caselist.append( v_cases[i] )
        for caseId in caselist:
            dat = Manyo.ElementContainerMatrix()
            if not GNH.SetElementContainerMatrix( dat, caseId ):
                msg = " SetElementContainerMatrix failed."
                raise UserWarning(msg)
            if mode_no == 5:
                # add periods infor of multi-runNos
                periods = Manyo.MakeDoubleVector()
                for a_runNo in runNo_list:
                    v_runno = mu.FindRunDataFolders(int(a_runNo))
                    target = "SAS{:06d}_10_254_000.edb".format(int(a_runNo))
                    for k in range(v_runno.size()):
                        fname = os.path.join(v_runno[k], target)
                        print("#[inamura 230531] fname={}".format(fname))
                        if os.path.exists(fname):
                            TT = mu.T0TreatToolsNeunet()
                            TT.readOrgEvent(fname)
                            a_period = TT.putMeasPeriodFromT0()
                            del TT
                            for l in range(a_period.size()):
                                periods.append(a_period[l])
                dat.PutHeaderPointer().OverWrite(mu.UTSUSEMI_KEY_HEAD_MEASPERIOD, periods)
            dat_list.append(dat)
    else:
        pfiles=GNH.MakeTempInfoFiles()
        dat = GetHistogramRPMT( runNo, pfiles[0], pfiles[1], time_range )
        dat_list.append(dat)
    del GNH

    ## Create Lambda Vector for each EC
    for dat in dat_list:
        tt1 = ms.CreateLambVector( dat )
        tt1.Execute()
        del tt1

    ## Convertion from histogram to per_unit
    for dat in dat_list:
        tt2 = ms.ConvertToPoint(dat)
        tt2.Execute()
        del tt2

    ## Solid angle correction
    if (SolAngCor):
        for dat in dat_list:
            for i in range(dat.PutSize()):
                if (dat(i).PutHeaderPointer().PutString("TYPE")=="PSD"):
                    for j in range( dat(i).PutSize() ):
                        ec = dat(i,j)
                        sa = ec.PutHeaderPointer().PutDouble("PixelSolidAngle")
                        if sa==0.0:
                            raise UserWarning("Solid angle = 0 at %d,%d"%(i,j))
                        else:
                            ec.MulMySelf(1.0/sa)

            p_chk = mu.UtsusemiCheckDataProcess()
            p_chk.AddProcess( dat, "SolidAngleCorrection" )
            del p_chk

    if len(dat_list)==1:
        return dat_list[0]
    return dat_list

########################################
def GetHistogramRPMT( runNo, wfile, dfile, timeslice=[] ):
    """
    Load EventData to produce histogram
    GetHistogramSAS( runNo=0, AxType="lambda", Xrange="0.7 7.7 0.05", Bank="SM", TimeSlice="-1 -1", frameBoundary=0.0, isTimeFoc=False )

    @param runNo     (int)     Run Number
    @retval DAT (ElementContainerMatrix)
    """

    dataPath = mu.UtsusemiEnvGetDataDir()
    t0Path = ""

    if type(runNo)==int:
        runNo_list = [ str(runNo) ]
    elif type(runNo)==str:
        runNo_list = runNo.split(" ")
        if len(runNo_list)==0:
            raise UserWarning("RunNo is invalid")
    else:
        raise UserWarning("RunNo is invalid")

    v_runNo = Manyo.MakeUInt4Vector()
    for a_runNo in runNo_list:
        v_runNo.append( int(a_runNo) )

    tt=mu.UtsusemiEventDataConverterRPMT()
    tt.LoadParamFiles( wfile, dfile )
    tt.SetHistAllocation()

    dat=Manyo.ElementContainerMatrix()
    tt.SetElementContainerMatrix( dat, v_runNo, dataPath, t0Path )

    Phi=mu.PhiCalculation(dat,0.2,0.2,0.2)
    Phi.CalcPhi()
    del Phi

    dat2 = Manyo.ElementContainerMatrix(dat.PutHeader())
    hh_dat2 = dat2.PutHeaderPointer()

    vid = Manyo.MakeInt4Vector()
    vid.push_back(SASBankDic["RPMT"])
    vnm = Manyo.MakeStringVector()
    vnm.push_back("RPMT")
    vbs = Manyo.MakeInt4Vector()
    vbs.push_back(820)

    hh_dat2.Add("BANKIDLIST", vid )
    hh_dat2.Add("BANKNAMELIST", vnm )
    hh_dat2.Add("BANKSIZELIST", vbs )

    num_of_eca = dat.PutSize()
    for i in range(num_of_eca):
        eca = dat(num_of_eca-1-i)

        hh = eca.PutHeader()
        #DETID=hh.PutInt4("DETID")
        #hh.OverWrite("DETID", num_of_eca-1-DETID)
        hh.Add("BANKID",SASBankDic["RPMT"])
        hh.Add("INDEXINBANK", i )
        eca2 = Manyo.ElementContainerArray(hh)
        num_of_ec = eca.PutSize()
        for j in range(num_of_ec):
            eca2.Add( eca(num_of_ec-1-j) )
        dat2.Add(eca2)
        """
        hh = eca.PutHeaderPointer()
        hh.Add("BANKID",SASBankDic["RPMT"])
        hh.Add("INDEXINBANK", i )
        dat2.Add(eca)
        """
    for i in range(520):
        eca2 = Manyo.ElementContainerArray()
        eca2.AddToHeader("DETID", i+520 )
        eca2.AddToHeader("MASKED",1 )
        dat2.Add(eca2)
    return dat2

########################################
def GetHistogramSAS( runNo=0, Xrange="0.7 7.6 0.1 dL", Bank="SM", TimeSlice="-1 -1", frameBoundary="0.0", isTimeFoc=False, SolAngCor=True, paramFiles="- -" ):
    """
    Load EventData to produce histogram
    GetHistogramSAS( runNo=0, AxType="lambda", Xrange="0.7 7.7 0.05", Bank="SM", TimeSlice="-1 -1", frameBoundary=0.0, isTimeFoc=False )

    @param runNo     (int)     Run Number
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param Bank      (string)  "SM"...  Monitors by "MON1" or "MON2"
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (string) "frameBoundary" or "frameBoundary TofMask_min TofMask_max"
    @param isTimeFoc (bool)    True or False
    @param SolAngCor  (bool)    True or False for Solid Angle Correction
    @param paramFiles (string) "WiringInfo DetectorInfo"
    @retval DAT (ElementContainerMatrix)
    """
    return GetHistogramBaseSAS( runNo, Xrange, Bank, TimeSlice, frameBoundary, isTimeFoc, SolAngCor, paramFiles, isPH=False, PHrange="" )

########################################
def GetMonHistSAS( runNo="0", MonNo=1, AxType="lambda", Xrange="0.7 7.6 0.1", MonIndex=0, TimeSlice="-1 -1", frameBoundary="0.0", CalibEffi=True, RPMTarea="150 150 152 152", RPMTeffiParam="0.10516 0.04838", paramFiles="- -" ):
    """
    Load EventData to produce histogram of monitors
    GetMonHistSAS( runNo="0", MonNo=1, AxType="lambda", Xrange="0.7 7.7 0.05", MonIndex=0, TimeSlice="-1 -1", frameBoundary="0.0", CalibEffi=True, RPMTarea="150 150 152 152", RPMTeffiParam="0.10516 0.04838" )

    @param runNo     (string)     Run Number
    @param MonNo     (int)     Monitor Number 1 (up-stream), 2 (down-stream), 3 or 10
    @param AxType    (string)  Type must be "tof', "lambda", "lamda2", "d" or "rd"
    @param Xrange    (string)  "startX endX widthX" separated with space (not comma)
    @param MonIndex  (int)
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (string) "frameBoundary" or "frameBoundary TofMask_min TofMask_max"
    @param CalibEffi (bool)    Monitor-detector efficiency calibration
    @param RPMTarea  (string)  Summation area of RPMT
    @param RPMTeffiParam (string) Efficiency parameters alpha and beta for RPMT
    @param paramFiles (stirng) "<wiringinfo> <detectroinfo>[ <caseinfo>]"
    @retval EC (ElementContainer)
    """

    Bank=""
    if MonNo==1:
        Bank="MON1"
    elif MonNo==2:
        Bank="MON2"
    elif MonNo==3:
        Bank="MON3"
    elif MonNo==10:
        Bank="RPMT"
    else:
        raise UserWarning("MonNo is invalid")

    ax_axis=AxType.lower()
    if ax_axis=="tof":
        Xrange += " tof"
    elif ax_axis=="lambda":
        Xrange += " dl"
    elif ax_axis=="lambda2":
        Xrange += " dl/l"
    elif ax_axis=="d":
        Xrange += " d"
    elif ax_axis=="rd":
        Xrange += " rd"
    else:
        raise UserWarning("AxType is invalid %s"%(AxType))

    dat_list = GetHistogramBaseSAS( runNo, Xrange, Bank, TimeSlice, frameBoundary, isTimeFoc=False, SolAngCor=False, paramFiles=paramFiles, isPH=False, PHrange="" )

    if type(dat_list)!=list:
        dat_list = [dat_list]
    ec_list = []
    for dat in dat_list:
        if Bank=="RPMT":
            eff_ab = RPMTeffiParam.split(" ")
            try:
                alpha = float(eff_ab[0])
                beta = float(eff_ab[1])
            except:
                raise UserWarning("RPMTeffiParam is invalid (%s)"%(eff_ab))
            if (CalibEffi):
                CBC.CorrectDetEffiRPMT( dat, alpha, beta )
            #return GetSumPixelArea( dat, RPMTarea )
            ec_list.append(GetSumPixelArea( dat, RPMTarea ))

        else:
            ec = Manyo.ElementContainer( dat.Put(MonIndex).Put(0) )
            #key_kc = "KICKERCOUNT"
            #kick_count = dat.PutHeader().PutInt4(key_kc)
            #ec.AddToHeader(key_kc,kick_count)
            ec.InputHeader( dat.PutHeader() )

            # Monitor Detector Efficienty Calibration
            if (CalibEffi):
                p_N2 = 0.465  #[atm]
                t_Mon = 12   # thickness of monitor [mm]
                t_Al  = 0.5  # thickness of Al window [mm]

                dec = ms.DetectorEffiCorrection()
                dec.N2MonEffiCorrection( ec, p_N2, t_Mon, t_Al )
                del dec
            #del dat
            #return ec
            ec_list.append(ec)
    if len(ec_list)==1:
        return ec_list[0]
    else:
        return ec_list

########################################
def GetPulseHeightSAS( runNo="0", Xrange="0 10000 10", Bank="SM", TimeSlice="-1 -1", frameBoundary="0.0", paramFiles="- -",PHbin=5 ):
    """
    Make Pulse Height data of EventData
    GetPulseHeightSAS( runNo=0, Xrange="0 10000 10", Bank="SM", TimeSlice="-1 -1", paramFiles="- -" ):

    @param runNo     (string)     Run Number
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param Bank      (string)  "SM"...  Monitors by "MON1" or "MON2"
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (string) "frameBoundary" or "frameBoundary TofMask_min TofMask_max"
    @param paramFiles (string) "WiringInfo DetectorInfo"
    @param PHbin      (int)    Pulse Height bin
    @retval ECA (ElementContainerArray)
    """
    ss = Xrange.split(" ")
    if len(ss)==3:
        Xrange = Xrange+" tof"

    isTimeFoc=False
    SolAngCor=False
    isPH=True
    PHrange=""
    dat = GetHistogramBaseSAS( runNo, Xrange, Bank, TimeSlice, frameBoundary, isTimeFoc, SolAngCor, paramFiles, isPH, PHrange )

    ret = BC.GetPH( dat, PHbin )
    pId_list = []
    for i in range( ret.PutSize() ):
        hh = ret(i).PutHeaderPointer()
        pId_list.append( hh.PutInt4("DetId") )
    pId_list.sort()

    eca = Manyo.ElementContainerArray(dat.PutHeader())
    SIH = Manyo.SearchInHeader()
    SIH.SetTarget( ret )
    for pId in pId_list:
        #print "pid=",pId
        if (SIH.Search( "DetId", pId )):
            rslt_v0 = SIH.PutResultIndex(0)
            tmp_ec = ret(rslt_v0[0])
            tmp_hh = tmp_ec.PutHeaderPointer()
            x_range = Manyo.MakeDoubleVector()
            x_range.push_back( float( pId-0.5 ) )
            x_range.push_back( float( pId+0.5 ) )
            tmp_hh.Add( "XRANGE", x_range )
            tmp_hh.Add( "isHistogram", 0 ) ##[inamura 150515]
            eca.Add( tmp_ec )
        else:
            print("serching failed at pId=",pId)
    del SIH
    del dat

    return eca


########################################
def GetMonHistByPhSAS( runNo="0", MonNo=1, Xrange="0.0 40000.0 10.0 tof", phRange="0 10000", MonIndex=0, TimeSlice="-1 -1", frameBoundary=0.0, CalibEffi=True ):
    """
    Load EventData to produce histogram of monitors limited by pulse height
    GetMonHistByPhSAS( runNo="0", MonNo=1, Xrange="0.0 40000.0 10.0 tof", phRange="0 10000", MonIndex=0, TimeSlice="-1 -1", frameBoundary=0.0, CalibEffi=True )

    @param runNo     (int)     Run Number
    @param MonNo     (int)     Monitor Number 1 (up-stream) or 2 (down-stream)
    @param Xrange    (string)  "startX endX widthX type" separated with space (not comma) : type=[dL, dL/L, tof]
    @param phRange   (string)  "lld hld"
    @param MonIndex  (int)     0.. digital  1.. analog
    @param TimeSlice (string)  "startTime endTime" separated with space: "-1 -1" means no-setting.
    @param frameBoundary (float)
    @param CalibEffi (bool)    Monitor-detector efficiency calibration
    @retval EC (ElementContainer)
    """
    Bank=""
    if MonNo==1:
        Bank="MON1"
    elif MonNo==2:
        Bank="MON2"
    elif MonNo==3:
        Bank="MON3"
    else:
        raise UserWarning("MonNo is invalid")

    dat = GetHistogramBaseSAS( runNo, Xrange, Bank, TimeSlice, str(frameBoundary), isTimeFoc=False, SolAngCor=False, paramFiles="- -", isPH=True, PHrange=phRange )

    ec = dat.Put(MonIndex).Put(0)
    key_kc = "KICKERCOUNT"
    kick_count = dat.PutHeader().PutInt4(key_kc)
    ec.AddToHeader(key_kc,kick_count)

    # Monitor Detector Efficienty Calibration
    if (CalibEffi):
        p_N2 = 0.465  #[atm]
        t_Mon = 12   # thickness of monitor [mm]
        t_Al  = 0.5  # thickness of Al window [mm]

        dec = ms.DetectorEffiCorrection()
        dec.N2MonEffiCorrection( ec, p_N2, t_Mon, t_Al )
        del dec



    return ec

#########################################
def MergeSMSC4DetMap(dat):
    """
    Merge Data SM and SC bank into one ElementContainerMatrix
    to plot using DetectMap
    @param dat  (ElementContainerMatrix)
    @retval ElementContainerMatrix
    """
    SIH = Manyo.SearchInHeader( dat )
    matrix = []
    for i in range(25):
        a_line=[]
        for j in range(125):
            a_line.append(None)
        matrix.append(a_line)

    ##SC_Lower
    pixel_no=66
    for det_id in range( 872, 928 ):
        if SIH.SearchArray("DETID", det_id ):
            vv = SIH.PutResultIndex(0)
            if vv.size()!=0:
                for i in range(25):
                    matrix[24-i][pixel_no] = (vv[0],i)
            pixel_no+=1

    ##SC_Upper
    pixel_no = 2
    for det_id in range( 816, 872 ):
        if SIH.SearchArray("DETID", det_id ):
            vv = SIH.PutResultIndex(0)
            if vv.size()!=0:
                for i in range(25):
                    matrix[24-i][pixel_no] =(vv[0],i)
            pixel_no+=1

    ##SC_Left
    det_no = 7
    for det_id in range( 928, 936 ):
        if SIH.SearchArray("DETID", det_id ):
            vv = SIH.PutResultIndex(0)
            if vv.size()!=0:
                for i in range(8):
                    #matrix[det_no][63-i] =(vv[0],i+8)
                    #matrix[det_no][65-i] =(vv[0],i+8)
                    matrix[det_no][65-i] =(vv[0],15-i)
            det_no=det_no-1

    ##SC_rigth
    det_no = 24
    for det_id in range( 936, 944 ):
        if SIH.SearchArray("DETID", det_id ):
            vv = SIH.PutResultIndex(0)
            if vv.size()!=0:
                for i in range(8):
                    #matrix[det_no][63-i] = (vv[0],i+8)
                    #matrix[det_no][65-i] = (vv[0],i+8)
                    matrix[det_no][65-i] =(vv[0],15-i)
            det_no=det_no-1


    ec_tmp = dat(0,0)
    xx=ec_tmp.PutXList()
    yy=ec_tmp.PutYList()
    ee=ec_tmp.PutEList()
    xkey=ec_tmp.PutXKey()
    ykey=ec_tmp.PutYKey()
    ekey=ec_tmp.PutEKey()
    yy_zero = [0]*len(yy)
    ee_zero = [0]*len(yy)
    hh_ec = Manyo.HeaderBase()
    hh_ec.Add("MASKED",1)
    ec_empty = Manyo.ElementContainer(hh_ec)
    ec_empty.Add(xkey,xx)
    ec_empty.Add(ykey,yy_zero)
    ec_empty.Add(ekey,ee_zero)
    ec_empty.SetKeys(xkey,ykey,ekey)

    hh=dat.PutHeader()
    hh.Erase("BANKIDLIST")
    hh.Erase("BANKSIZELIST")
    hh.Erase("BANKNAMELIST")
    """
    v_id=Manyo.MakeInt4Vector(3)
    v_id[0]=0
    v_id[1]=1
    v_id[2]=2
    v_size=Manyo.MakeInt4Vector(3)
    v_size[0]=112
    v_size[1]=25
    v_size[2]=144
    v_name=Manyo.MakeStringVector(3)
    v_name[0]="SM16"
    v_name[1]="SC"
    v_name[2]="SM14"
    """
    v_id=Manyo.MakeInt4Vector(1)
    v_id[0]=0
    v_size=Manyo.MakeInt4Vector(1)
    v_size[0]=112+25+144
    v_name=Manyo.MakeStringVector(1)
    v_name[0]="SM+SC"

    hh.Add("BANKIDLIST",v_id)
    hh.Add("BANKSIZELIST",v_size)
    hh.Add("BANKNAMELIST",v_name)

    ecm = Manyo.ElementContainerMatrix( hh )


    # SM16
    for psd_no in range(112):
        psd_id = 280 + psd_no
        if SIH.SearchArray("DETID", psd_id ):
            vv = SIH.PutResultIndex(0)
            if len(vv)!=0:
                eca = dat.Put(vv[0])
                hh = eca.PutHeaderPointer()
                hh.Erase("BANKID")
                hh.Erase("INDEXINBANK")
                hh.Add("BANKID",0)
                hh.Add("INDEXINBANK", psd_no)

                ecm.Add( eca )
                del eca
            else:
                eca = Manyo.ElementContainerArray()
                hh=eca.PutHeaderPointer()
                hh.Add("DETID", psd_id)
                hh.Add("MASKED", 1 )
                hh.Add("BANKID", 0 )
                hh.Add("INDEXINBANK", psd_no )
                ecm.Add(eca)
                del eca

    # SC
    det_no = 112
    for a_psd in matrix:
        det_id = 2000 + det_no
        eca = Manyo.ElementContainerArray()
        hh=eca.PutHeaderPointer()
        hh.Add("DETID", det_id)
        hh.Add("BANKID", 0 )
        hh.Add("INDEXINBANK", det_no )
        isAllNone = True
        pixelNo=0
        for a_pixel in a_psd:

            if a_pixel==None:
                eca.Add( ec_empty )
            else:
                #print "det_id,pixelNo=",det_no,pixelNo
                eca.Add( dat(a_pixel[0],a_pixel[1]) )
                #hh_ec=dat(a_pixel[0]).PutHeaderPointer()
                #print "DetId,pixelNo=",hh_ec.PutInt4("DETID")
                isAllNone = False
            pixelNo+=1
        if isAllNone:
            hh.Add("MASKED", 1 )
        else:
            hh.Add("MASKED", 0 )
        ecm.Add(eca)
        det_no +=1

    # SM14
    for psd_no in range(144):
        psd_id = 392 + psd_no
        if SIH.SearchArray("DETID", psd_id ):
            vv = SIH.PutResultIndex(0)
            if len(vv)!=0:
                eca = dat.Put(vv[0])
                hh = eca.PutHeaderPointer()
                hh.Erase("BANKID")
                hh.Erase("INDEXINBANK")
                hh.Add("BANKID",0)
                hh.Add("INDEXINBANK", 112+25+psd_no)
                ecm.Add( eca )
                del eca
            else:
                eca = Manyo.ElementContainerArray()
                hh=eca.PutHeaderPointer()
                hh.Add("DETID", psd_no+112+25)
                hh.Add("MASKED", 1 )
                hh.Add("BANKID", 0 )
                hh.Add("INDEXINBANK", 112+25+psd_no)
                ecm.Add(eca)
                del eca

    del SIH
    return ecm

def RPMTRemoveCenterLine( dat, PsdRange="200 201", PixelRange="198 199" ):
    """
    Merge Data SM and SC bank into one ElementContainerMatrix
    to plot using DetectMap
    @param dat  [Def:DAT] (ElementContainerMatrix)
    @param PsdRange
    @param PixelRange
    @retval None
    """
    import Manyo

    psd_range_list=PsdRange.split(" ")
    if len(psd_range_list)!=2:
        raise UserWarning("PsdRange is invalid #1")
    psd1 = int(psd_range_list[0])
    psd2 = int(psd_range_list[1])

    pixel_range_list=PixelRange.split(" ")
    if len(pixel_range_list)!=2:
        raise UserWarning("PixelRange is invalid")
    pixel1 = int(pixel_range_list[0])
    pixel2 = int(pixel_range_list[1])

    SIH = Manyo.SearchInHeader()
    SIH.SetTarget( dat )
    eca1 = None
    eca2 = None
    if (SIH.SearchArray( "DETID", psd1 )):
        rslt = SIH.PutResultIndex(0)
        if rslt.size()!=0:
            eca1 = dat.PutPointer(rslt[0])
    if (SIH.SearchArray( "DETID", psd2 )):
        rslt = SIH.PutResultIndex(0)
        if rslt.size()!=0:
            eca2 = dat.PutPointer(rslt[0])

    if eca1==None or eca2==None:
        raise UserWarning("PsdRange is invalid #2")

    import math
    for i in range( eca1.PutSize() ):
        ec1 = eca1(i)
        ec2 = eca2(i)
        keyX = ec1.PutXKey()
        keyY = ec1.PutYKey()
        keyE = ec1.PutEKey()

        yy1 = ec1.PutY()
        yy2 = ec2.PutY()
        ee1 = ec1.PutE()
        ee2 = ec2.PutE()
        yy_ave = Manyo.MakeDoubleVector()
        ee_ave = Manyo.MakeDoubleVector()
        for j in range(yy1.size()):
            yy_ave.push_back( (yy1[j]+yy2[j])/2.0 )
            ee_ave.push_back( math.sqrt( (ee1[j]*ee1[j])+(ee2[j]*ee2[j]) ) )
        ec1.Replace( keyY, yy_ave )
        ec1.Replace( keyE, ee_ave )
        ec2.Replace( keyY, yy_ave )
        ec2.Replace( keyE, ee_ave )

        ec1.SetKeys( keyX, keyY, keyE )
        ec2.SetKeys( keyX, keyY, keyE )

    #del SIH

    for i in range( dat.PutSize() ):
        if dat(i).PutSize()==0:
            continue
        ec1 = dat(i,pixel1)
        ec2 = dat(i,pixel2)
        """
        ec1 = None
        ec2 = None
        for j in range(dat(i).PutSize()):
            ypos = dat(i,j).PutHeaderPointer().PutInt4("YPOS")
            if ypos==pixel1:
                ec1 = dat(i,j)
            elif ypos==pixel2:
                ec2 = dat(i,j)
        if ec1==None or ec2==None:
            raise
        """
        keyX = ec1.PutXKey()
        keyY = ec1.PutYKey()
        keyE = ec1.PutEKey()

        yy1 = ec1.PutY()
        yy2 = ec2.PutY()
        ee1 = ec1.PutE()
        ee2 = ec2.PutE()
        yy_ave = Manyo.MakeDoubleVector()
        ee_ave = Manyo.MakeDoubleVector()
        for j in range(yy1.size()):
            yy_ave.push_back( (yy1[j]+yy2[j])/2.0 )
            ee_ave.push_back( math.sqrt( (ee1[j]*ee1[j])+(ee2[j]*ee2[j]) ) )
        ec1.Replace( keyY, yy_ave )
        ec1.Replace( keyE, ee_ave )
        ec2.Replace( keyY, yy_ave )
        ec2.Replace( keyE, ee_ave )

        ec1.SetKeys( keyX, keyY, keyE )
        ec2.SetKeys( keyX, keyY, keyE )

    del SIH


def ConvertFromRPMTK2Psd(runNo, daqId, modNo=0, dataPath=None, modeNo=5, rpmtPrm=[128,1024,128,1024], lmtCoi=32, dfile=""):
    """
    Convert event data for RPMT of KEK to ones for PSD

    @param runNo (int) runNo
    @param daqId (int) Daq Id for RPMT of KEK
    @param dataPath (string) Data path
    @param modNo (int) Module No. for RPMT of KEK
    @param rpmtPrm (list)
    @param lmtCoi (int)
    @param dfile (string) DetectorInfo file
    @retval None
    """
    print( ".... Start Conversion event data from RPMT-KEK to PSD .... " )
    print("ConvertFromRPMTK2Psd >>> LLD_X={:d}, HLD_X={:d}, LmtCoi={:d}".format(rpmtPrm[0], rpmtPrm[1], lmtCoi))
    print("ConvertFromRPMTK2Psd >>> LLD_Y={:d}, HLD_Y={:d}, LmtCoi={:d}".format(rpmtPrm[2], rpmtPrm[3], lmtCoi))
    root_data = mu.FindRootDataFolder()
    inst_code = mu.UtsusemiEnvGetInstCode()
    run_folders_v = mu.FindRunDataFolders( runNo )
    target_dir_list = None
    for i in range(run_folders_v.size()):
        # print(" run_folders_v = ", run_folders_v[i])
        run_folder = run_folders_v[i]
        target_dir = os.path.join(run_folder, "%3s%06d_%02d_%03d_???.edb"%(inst_code, runNo,daqId, modNo))
        target_dir_list = glob.glob(target_dir)
        if len(target_dir_list) != 0:
            break
        else:
            target_dir_list = None
    if target_dir_list is None:
        raise UserWarning( "ConvertFromRPMTK2Psd >> Not found RunNo in %s"%(target_dir) )
    target_dir_list.sort()
    Output_dir = os.path.dirname( target_dir_list[0] )
    RPMT_data_files = Manyo.MakeStringVector()
    for a_file in target_dir_list:
        RPMT_data_files.append( str( a_file ) )


    if dataPath!=None:
        Output_dir = os.path.join( root_data, dataPath, os.path.basename( Output_dir ) )
        if not os.access( Output_dir, os.F_OK ):
            try:
                os.system( "mkdir -p %s"%(Output_dir) )
            except:
                raise UserWarning( "ConvertFromRPMTK2Psd >> Cannot make directory as %s"%(Output_dir) )
        else:
            try:
                if mu.UtsusemiEnvGetDebugMode():
                    print( "ConvertFromRPMTK2Psd >>> Try to Delete Old Data" )
                os.system( "rm -rf %s/*"%(Output_dir) )
                if mu.UtsusemiEnvGetDebugMode():
                    print( "Delete OK." )
            except:
                raise UserWarning( "ConvertFromRPMTK2Psd >> Cannot delete old files in %s"%(Output_dir) )

    if not os.access( Output_dir, os.W_OK):
        raise UserWarning( "ConvertFromRPMTK2Psd >> Cannot write converted data in %s"%(Output_dir) )

    tt=ms.RPMTDataConverter()
    tt.SetDaqId(daqId)
    if tt.SetParamsFromAnaEnv(runNo, modeNo, "", dfile):
        tt.Execute(RPMT_data_files, Output_dir, runNo, rpmtPrm[0], rpmtPrm[1], rpmtPrm[2], rpmtPrm[3], lmtCoi)
    else:
        print("ConvertFromRPMTK2Psd >>> RPMTDataConverter::SetParamFromAnaEnv false")
    del tt

#########################################
def GetPulseHeightFromRPMTK( runNo, daqId, modNo=0, dataPath=None, width_bin=2, max_bin=4048, LmtCoin=32):
    """
    Get PulseHeight data from raw data from RPMT of KEK

    @param runNo (int) runNo
    @param daqId (int) Daq Id for RPMT of KEK
    @param modNo (int) Module No. for RPMT of KEK
    @param dataPath (string)
    @param width_bin (int) bin width of Pulse Height channels
    @param max_bin (int) maximum bin of Pulse Height channels
    @param LmtCoin (int) Upper limit of Coincidence clock
    @retval ECA ElementContainerArray of pulse height histograms for X, Y, X+Y
    """
    print( ".... Create Pulse Height data from event data from RPMT-KEK .... " )
    root_data = mu.FindRootDataFolder()
    inst_code = mu.UtsusemiEnvGetInstCode()
    run_folders_v = mu.FindRunDataFolders( runNo )
    run_folder = run_folders_v[0]
    target_dir = os.path.join( run_folder, "%3s%06d_%02d_%03d_???.edb"%(inst_code,runNo,daqId, modNo) )
    target_dir_list = glob.glob( target_dir )
    if len(target_dir_list)==0:
        raise UserWarning( "ConvertFromRPMTK2Psd >> Not found RunNo in %s"%(target_dir) )
    target_dir_list.sort()
    Output_dir = os.path.dirname( target_dir_list[0] )
    RPMT_data_files = Manyo.MakeStringVector()
    for a_file in target_dir_list:
        RPMT_data_files.append( str( a_file ) )

    tt=ms.RPMTDataConverter()
    tt.SetDaqId(daqId)
    HH=Manyo.HeaderBase()
    HH.Add(mu.UTSUSEMI_KEY_HEAD_RUNNUMBER, "%3s%06d"%(inst_code,runNo))
    HH.Add(mu.UTSUSEMI_KEY_HEAD_INSTRUMENT, inst_code)
    ecX=Manyo.ElementContainer(HH)
    ecY=Manyo.ElementContainer(HH)
    tt.PutPulseHeight(ecX, ecY, RPMT_data_files, width_bin, max_bin, LmtCoin)
    ec = ecX+ecY
    ECA=Manyo.ElementContainerArray(HH)
    ECA.Add(ecX)
    ECA.Add(ecY)
    ECA.Add(ec)

    del tt
    return ECA
#########################################
def RPMTGetBeamCenterDropPosi(DAT=None, lamStep="1.0,2.0,3.0,4.0,5.0,6.0,7.0", deltaLam=0.5, init_par="1.0e14, 5.0, 5.0, 150.0, 150.0", factor=1e-10, withPlot=False):
    """
    Get beam center positions at each lambda range
    RPMTGetBeamCenterDropPosi(DAT=None, lamStep="1.0,2.0,3.0,4.0,5.0,6.0,7.0", deltaLam=0.5, init_par="1.0e14, 5.0, 5.0, 150.0, 150.0", factor=1e-10, withPlot=False)

    @param DAT (ElementContainerMatrix)
    @param lamStep (string) list of the center lambda
    @param deltaLam (float) width of lambra range for each the center lambda
    @param init_par (string) Initial values for fitting "<A>, <w_X>, <w_Y>, <c_X>, <c_Y>"
    @param factor (float) Intensity = factor*Intensity
    @param withPlot (bool) Plot results or not
    @retval EC ElementContainer for X-Y positions of beam center
    """
    DATs = DAT.Mul(factor)
    lamStep = lamStep.strip()
    num_space = len(lamStep.split(" "))
    num_comma = len(lamStep.split(","))
    lam_step = []
    if num_space > num_comma:
        for l in lamStep.split(" "):
            lam_step.append(float(l))
    elif num_comma >= num_space:
        LamRnage = lamStep.replace(" ","")
        for l in lamStep.split(","):
            lam_step.append(float(l))
    else:
        raise UserWarning("RPMTGetBeamCenterDropPosi>> invalid Lambda range parameter")

    init_par=init_par.strip()
    num_space = len(init_par.split(" "))
    num_comma = len(init_par.split(","))
    init_param = []
    if num_comma == 5:
        for l in init_par.split(","):
            init_param.append(float(l))
    elif num_space == 5:
        for l in init_par.split(" "):
            init_param.append(float(l))
    else:
        raise UserWarning("RPMTGetBeamCenterDropPosi>> invalid Lambda range parameter")

    param_initial = tuple(init_param)
    print("param_initial = ",param_initial)

    import numpy as np
    import math
    from scipy.optimize import curve_fit
    def gauss_2d(X, A, sigma_x, sigma_y, mu_x, mu_y):
        x, y = X
        z = A * np.exp(-((x-mu_x)**2)/(2.0 * sigma_x**2)) * np.exp(-((y-mu_y)**2)/(2.0 * sigma_y**2))
        return z.ravel()
    def line_1d(X, A, B):
        return A + (X * B)
    def drop_lambda(X, A, B):
        return A - (B * X * X)

    # Data
    num_x = DATs.PutSize()
    num_y = DATs(0).PutSize()
    xx = np.arange(num_x)
    yy = np.arange(num_y)

    x_data, y_data = np.meshgrid(xx, yy)
    z_data = np.zeros((num_y, num_x), np.float64)

    mu_x_list = []
    mu_y_list = []
    mu_e_list = []
    for lr in lam_step:
        # Make data
        for i in range(num_x):
            for j in range(num_y):
                EC = DATs(i, j)
                p = EC.Sum((lr - deltaLam), (lr + deltaLam))
                z_data[j, i] = p.first

        # Fitting
        #param_initial = ( 1000, 5.0, 5.0, 150.0, 150.0 )
        X = np.array([x_data, y_data])
        popt, pcov = curve_fit(gauss_2d, X, z_data.ravel(), p0=param_initial)
        perr = np.sqrt(np.diag(pcov))

        # Results
        A, sigma_x, sigma_y, mu_x, mu_y = popt
        print("perr= {}".format(perr))
        print(" A = {}\n sigma_x = {}\n sigma_y={}\n mu_x={}\n mu_y={}".format(A, sigma_x, sigma_y, mu_x, mu_y))
        DI=ms.SASDetectorInfoEditorRPMT()
        mu_x_list.append(float(mu_x) * DI.PutPixelSizeX())
        mu_y_list.append(float(mu_y) * DI.PutPixelSizeY())
        #mu_x_list.append(float(mu_x))
        #mu_y_list.append(float(mu_y))
        mu_e_list.append(1.0)

    print("mu_x_list = ",mu_x_list)
    print("mu_y_list = ",mu_y_list)

    EC = Manyo.ElementContainer()
    EC.Add("Xch", mu_x_list, "mm")
    EC.Add("Ych", mu_y_list, "mm")
    EC.Add("Error", mu_e_list, "mm")
    EC.SetKeys("Xch", "Ych", "Error")

    # line fit
    popt, pcov = curve_fit(line_1d, mu_x_list, mu_y_list)
    perr = np.sqrt(np.diag(pcov))
    A, B = popt
    print(type(A),type(B))
    A=float(A)
    B=float(B)

    print("Line Fitting: A={}, B={}".format(A, B))
    #rotAng = math.atan(1.0 / (float(B) * 0.207 / 0.205)) / math.pi * 180.0
    rotAng = math.atan(1.0 / B) / math.pi * 180.0
    print("Rotate Angle = {}".format(rotAng))

    x_fit = []
    y_fit = []
    e_fit = []
    for x in range(100):
        #xx = 150.5 + (0.01 * float(x))
        xx = 30.75 + (0.01 * float(x))
        x_fit.append(xx)
        y_fit.append(A + (B * xx))
        e_fit.append(1.0)

    # print(x_fit)
    # print(y_fit)
    EC2 = Manyo.ElementContainer()
    EC2.Add("Xch", x_fit, "mm")
    EC2.Add("Ych", y_fit, "mm")
    EC2.Add("Error", e_fit, "mm")
    EC2.SetKeys("Xch", "Ych", "Error")

    x_p = []
    y_p = []
    e_p = []
    print("Line Fitting: A={}, B={}".format(A, B))
    for x1, y1 in zip(mu_x_list, mu_y_list):
        x0 = (x1 + (B * y1) - (A * B)) / ((B * B) + 1)
        y0 = A + (x0 * B)
        x_p.append(x0)
        y_p.append(y0)
        e_p.append(0.10)
        # print("x1 {} -> {}".format(x1, x0))
        # print("y1 {} -> {}".format(y1, y_p[-1]))
    EC3 = Manyo.ElementContainer()
    EC3.Add("Xch", x_p, "mml")
    EC3.Add("Ych", y_p, "mm")
    EC3.Add("Error", e_p, "mm")
    EC3.SetKeys("Xch", "Ych", "Error")

    # Dropping distance
    drop_dist = []
    drop_err = []
    for xp, yp in zip(x_p, y_p):
        dist = -1.0 *  math.sqrt(((xp - x_p[0]) * (xp - x_p[0])) + ((yp - y_p[0]) * (yp - y_p[0])))
        drop_dist.append(dist)
        drop_err.append(0.1)

    popt, pcov = curve_fit(drop_lambda, lam_step, drop_dist)
    perr = np.sqrt(np.diag(pcov))
    A, B = popt

    A = float(A)
    B = float(B)
    print( "Drop fitting A = {}, B= {}".format(A, B))
    drop_lam_fit = []
    drop_dist_fit = []
    drop_err_fit = []
    for i in range(160):
        drop_lam_fit.append(float(i)/10.0)
        drop_dist_fit.append(-1.0 * (B * float(i) * float(i) / 100.0)) # Distance=0 at lambda -> 0.
        drop_err_fit.append(0.1)

    EC4 = Manyo.ElementContainer()
    EC4.Add("Lambda", lam_step, "Ang")
    EC4.Add("Distance", drop_dist, "mm")
    EC4.Add("Error", drop_err, "mm")
    EC4.SetKeys("Lambda", "Distance", "Error")
    EC5 = Manyo.ElementContainer()
    EC5.Add("Lambda", drop_lam_fit, "Ang")
    EC5.Add("Distance", drop_dist_fit, "mm")
    EC5.Add("Error", drop_err_fit, "mm")
    EC5.SetKeys("Lambda", "Distance", "Error")

    drop_origin_dx = x_p[-1] - x_p[0]
    drop_origin_dy = y_p[-1] - y_p[0]
    drop_origin_dl = math.sqrt((drop_origin_dx * drop_origin_dx) + (drop_origin_dy * drop_origin_dy))
    drop_origin_dx /= drop_origin_dl
    drop_origin_dy /= drop_origin_dl
    drop_origin_x = drop_origin_dx * math.fabs(A) + x_p[0]
    drop_origin_y = drop_origin_dy * math.fabs(A) + y_p[0]

    drop_origin_px = drop_origin_x / 0.205
    drop_origin_py = drop_origin_y / 0.207

    print("True Beam Center x = {}, y = {} [mm]".format(drop_origin_x, drop_origin_y))
    print("True Beam Center x = {}, y = {} [pixel]".format(drop_origin_px, drop_origin_py))

    # Save results into xml file
    # under construction

    if withPlot:
        import VisualModule as VB
        p1 = VB.MPlot([EC, EC2, EC3])
        p1 = VB.MPlot([EC4, EC5])
        # VB.M2PlotPlus(DAT)

    '''
    ec_lam = []
    ec_drop = []
    ec_e = []
    for i in range(len(mu_x_list)-1):
        inext = i + 1
        ec_lam.append((lam_step[i] + lam_step[inext]) / 2.0)
        mx = (mu_x_list[inext] - mu_x_list[i]) * 0.205
        my = (mu_y_list[inext] - mu_y_list[i]) * 0.207
        ec_drop.append(math.sqrt(mx * mx + my * my))
        ec_e.append(1.0)

    EC2 = Manyo.ElementContainer()
    EC2.Add("Lambda", ec_lam, "Ang")
    EC2.Add("Distance", ec_drop, "mm")
    EC2.Add("Error", ec_e, "mm")
    EC2.SetKeys("Lambda", "Distance", "Error")

    if withPlot:
        import VisualModule as VB
        p1 = VB.MPlot(EC)
        p2 = VB.MPlot(EC2)
        VB.M2PlotPlus(DAT)
    '''
    return EC

#########################################
def CreateRPMTDetectorInfoFromData(runNo=206316, lamStep="1.0,2.0,3.0,4.0,5.0,6.0,7.0", deltaLam=0.5, factor=1e-10, withPlot=False):
    """
    Create new detector info for RPMT from beam center positions drifted by a gravity
    CreateRPMTDetectorInfoFromData(runNo="206316", lamStep="1.0,2.0,3.0,4.0,5.0,6.0,7.0", deltaLam=0.5, factor=1e-10, withPlot=False)

    @param runNo (int)
    @param lamStep (string) list of the center lambda
    @param deltaLam (float) width of lambra range for each the center lambda
    @param factor (float) Intensity = factor*Intensity
    @param withPlot (bool) Plot results or not
    @retval None
    """

    dfile_org = os.path.join( mu.UtsusemiEnvGetBaseDir(), mu.UtsusemiEnvGetInstCode(), "ana", "Reduction", "DetectorInfo_newRPMT_origin.xml")
    DAT0=GetHistogramSAS( runNo=str(runNo), Xrange="0.7 15.5 0.1 dL", Bank="RPMT", TimeSlice="-1 -1", frameBoundary="0.0", isTimeFoc=False, SolAngCor=True, paramFiles="- " + dfile_org )
    DAT0.MulMySelf(factor)

    lamStep = lamStep.strip()
    num_space = len(lamStep.split(" "))
    num_comma = len(lamStep.split(","))
    lam_step = []
    if num_space > num_comma:
        for l in lamStep.split(" "):
            lam_step.append(float(l))
    elif num_comma >= num_space:
        LamRnage = lamStep.replace(" ","")
        for l in lamStep.split(","):
            lam_step.append(float(l))
    else:
        raise UserWarning("CreateRPMTDetectorInfoFromData >>> invalid Lambda range parameter")

    import numpy as np
    import math
    from scipy.optimize import curve_fit
    def gauss_2d(X, A, sigma_x, sigma_y, mu_x, mu_y):
        x, y = X
        z = A * np.exp(-((x-mu_x)**2)/(2.0 * sigma_x**2)) * np.exp(-((y-mu_y)**2)/(2.0 * sigma_y**2))
        return z.ravel()
    def line_1d(X, A, B):
        return A + (X * B)
    def drop_lambda(X, A, B):
        return A - (B * X * X)

    # Find Beam Center to fix XchShift and YchShift
    DI=ms.SASDetectorInfoEditorRPMT()
    if DI.Read(dfile_org):
        pass
    else:
        print("Failed to Read {}".format(dfile_org))
        return False

    # Data
    def GetBeamCenter(DAT, lam_step, deltaLam):
        num_x = DAT.PutSize()
        num_y = DAT(0).PutSize()
        xx = np.arange(num_x)
        yy = np.arange(num_y)

        x_data, y_data = np.meshgrid(xx, yy)
        z_data = np.zeros((num_y, num_x), np.float64)

        mu_x_list = []
        mu_y_list = []
        mu_e_list = []
        for lr in lam_step:
            # Make data
            for i in range(num_x):
                for j in range(num_y):
                    EC = DAT(i, j)
                    p = EC.Sum((lr - deltaLam), (lr + deltaLam))
                    z_data[j, i] = p.first

            # max-value position
            iy_max, ix_max = np.unravel_index(np.argmax(z_data), z_data.shape)

            # Fitting
            #param_initial = ( 1000, 5.0, 5.0, 150.0, 150.0 )
            param_initial = ( 1000.0, 5.0, 5.0, ix_max, iy_max )
            X = np.array([x_data, y_data])
            popt, pcov = curve_fit(gauss_2d, X, z_data.ravel(), p0=param_initial)
            perr = np.sqrt(np.diag(pcov))

            # Results
            A, sigma_x, sigma_y, mu_x, mu_y = popt
            mu_x_list.append(float(mu_x) * DI.PutPixelSizeX())
            mu_y_list.append(float(mu_y) * DI.PutPixelSizeY())
            mu_e_list.append(1.0)

            # If Debug Mode
            if mu.UtsusemiEnvGetDebugMode():
                print("Fitting ix_max, iy_max = {}, {}".format(ix_max, iy_max))
                print("perr= {}".format(perr))
                print(" A = {}\n sigma_x = {}\n sigma_y={}\n mu_x={}\n mu_y={}".format(A, sigma_x, sigma_y, mu_x, mu_y))
        return (mu_x_list, mu_y_list, mu_e_list)

    mu_x_list, mu_y_list, mu_e_list = GetBeamCenter(DAT0, [1.5], 1.0)
    if mu.UtsusemiEnvGetDebugMode():
        print(mu_x_list[0]/DI.PutPixelSizeX(), mu_y_list[0]/DI.PutPixelSizeY())
        print("DI.PutSizeXch()={}".format(DI.PutSizeXch()))
        print("DI.PutSizeXch()/2.0 = {}".format(DI.PutSizeXch()/2.0))
    # _XchOffset = (UInt4)(_numOfLines/_binWidth/2 - (Int4)(_size_Xch/2) + _XchShift);
    # _YchOffset = (UInt4)(_numOfLines/_binWidth/2 - (Int4)(_size_Ych/2) + _YchShift);
    # PutNumOfLines();    UInt4 PutBinWidth();    UInt4 PutSizeXch();    UInt4 PutSizeYch();

    xchshift = int((DI.PutSizeXch()/2.0) -1.0 - mu_x_list[0]/DI.PutPixelSizeX()) # [pixel]
    ychshift = int((DI.PutSizeYch()/2.0) -1.0 - mu_y_list[0]/DI.PutPixelSizeY())
    if mu.UtsusemiEnvGetDebugMode():
        print("CreateRPMTDetectorInfoFromData >>> XchShift, YchShift = {}, {}".format(xchshift, ychshift))
    DI.SetChannelShift(xchshift, ychshift)
    tmp_dfile = os.path.join(os.getcwd(), "tmp_DetectorInfo_newRPMT.xml")
    if DI.Write(tmp_dfile):
        print("CreateRPMTDetectorInfoFromData >>> Temporal file = {}".format(tmp_dfile))
    else:
        raise

    # Find lambda dependency of Beam Center
    DAT1=GetHistogramSAS( runNo=runNo, Xrange="0.7 15.5 0.1 dL", Bank="RPMT", TimeSlice="-1 -1", frameBoundary="0.0", isTimeFoc=False, SolAngCor=True, paramFiles="- " + tmp_dfile )
    DAT1.MulMySelf(factor)
    mu_x_list, mu_y_list, mu_e_list = GetBeamCenter(DAT1, lam_step, deltaLam)
    if mu.UtsusemiEnvGetDebugMode():
        print("mu_x_list = ",mu_x_list)
        print("mu_y_list = ",mu_y_list)

    hh_ec = Manyo.HeaderBase()
    hh_ec.Add(mu.UTSUSEMI_KEY_HEAD_RUNNUMBER, "SAS{:06d}".format(runNo))
    hh_ec.Add(mu.UTSUSEMI_KEY_HEAD_LABEL, "Measured")
    EC = Manyo.ElementContainer(hh_ec)
    EC.Add("Xch", mu_x_list, "mm")
    EC.Add("Ych", mu_y_list, "mm")
    EC.Add("Error", mu_e_list, "mm")
    EC.SetKeys("Xch", "Ych", "Error")

    # line fit
    popt, pcov = curve_fit(line_1d, mu_x_list, mu_y_list)
    perr = np.sqrt(np.diag(pcov))
    A, B = popt

    A=float(A)
    B=float(B)

    if mu.UtsusemiEnvGetDebugMode():
        print("Line Fitting: A={}, B={}".format(A, B))

    rotAng = math.atan(1.0 / B) / math.pi * 180.0
    print("CreateRPMTDetectorInfoFromData >>> Rotate Angle = {}".format(rotAng))

    x_fit = []
    y_fit = []
    e_fit = []
    for x in range(100):
        #xx = 150.5 + (0.01 * float(x))
        xx = 30.75 + (0.01 * float(x))
        x_fit.append(xx)
        y_fit.append(A + (B * xx))
        e_fit.append(1.0)

    hh_ec.OverWrite(mu.UTSUSEMI_KEY_HEAD_LABEL, "Fitting")
    EC2 = Manyo.ElementContainer(hh_ec)
    EC2.Add("Xch", x_fit, "mm")
    EC2.Add("Ych", y_fit, "mm")
    EC2.Add("Error", e_fit, "mm")
    EC2.SetKeys("Xch", "Ych", "Error")

    x_p = []
    y_p = []
    e_p = []
    if mu.UtsusemiEnvGetDebugMode():
        print("Line Fitting: A={}, B={}".format(A, B))
    for x1, y1 in zip(mu_x_list, mu_y_list):
        x0 = (x1 + (B * y1) - (A * B)) / ((B * B) + 1)
        y0 = A + (x0 * B)
        x_p.append(x0)
        y_p.append(y0)
        e_p.append(0.10)

    hh_ec.OverWrite(mu.UTSUSEMI_KEY_HEAD_LABEL, "Positions")
    EC3 = Manyo.ElementContainer(hh_ec)
    EC3.Add("Xch", x_p, "mml")
    EC3.Add("Ych", y_p, "mm")
    EC3.Add("Error", e_p, "mm")
    EC3.SetKeys("Xch", "Ych", "Error")

    # Dropping distance
    drop_dist = []
    drop_err = []
    for xp, yp in zip(x_p, y_p):
        dist = -1.0 *  math.sqrt(((xp - x_p[0]) * (xp - x_p[0])) + ((yp - y_p[0]) * (yp - y_p[0])))
        drop_dist.append(dist)
        drop_err.append(0.1)

    popt, pcov = curve_fit(drop_lambda, lam_step, drop_dist)
    perr = np.sqrt(np.diag(pcov))
    A, B = popt

    A = float(A)
    B = float(B)
    print( "CreateRPMTDetectorInfoFromData >>> Drop fitting A = {}, B= {}".format(A, B))

    # Fitted line
    drop_lam_fit = []
    drop_dist_fit = []
    drop_err_fit = []
    for i in range(160):
        drop_lam_fit.append(float(i)/10.0)
        drop_dist_fit.append(-1.0 * (B * float(i) * float(i) / 100.0)) # Distance=0 at lambda -> 0.
        drop_err_fit.append(0.1)

    hh_ec.OverWrite(mu.UTSUSEMI_KEY_HEAD_LABEL, "Measured")
    EC4 = Manyo.ElementContainer(hh_ec)
    EC4.Add("Lambda", lam_step, "Ang")
    EC4.Add("Distance", drop_dist, "mm")
    EC4.Add("Error", drop_err, "mm")
    EC4.SetKeys("Lambda", "Distance", "Error")
    hh_ec.OverWrite(mu.UTSUSEMI_KEY_HEAD_LABEL, "Fitting")
    EC5 = Manyo.ElementContainer(hh_ec)
    EC5.Add("Lambda", drop_lam_fit, "Ang")
    EC5.Add("Distance", drop_dist_fit, "mm")
    EC5.Add("Error", drop_err_fit, "mm")
    EC5.SetKeys("Lambda", "Distance", "Error")

    drop_origin_dx = x_p[-1] - x_p[0]
    drop_origin_dy = y_p[-1] - y_p[0]
    drop_origin_dl = math.sqrt((drop_origin_dx * drop_origin_dx) + (drop_origin_dy * drop_origin_dy))
    drop_origin_dx /= drop_origin_dl
    drop_origin_dy /= drop_origin_dl
    drop_origin_x = drop_origin_dx * math.fabs(A) + x_p[0]
    drop_origin_y = drop_origin_dy * math.fabs(A) + y_p[0]

    drop_origin_px = drop_origin_x / 0.205
    drop_origin_py = drop_origin_y / 0.207

    print("CreateRPMTDetectorInfoFromData >> True Beam Center x = {}, y = {} [mm]".format(drop_origin_x, drop_origin_y))
    print("CreateRPMTDetectorInfoFromData >> True Beam Center x = {}, y = {} [pixel]".format(drop_origin_px, drop_origin_py))

    # Save results into xml file
    out_file = None
    if DI.Read(dfile_org):
        if DI.SetChannelShift(xchshift, ychshift):
            if DI.SetBeamCenterPixel(drop_origin_px, drop_origin_py):
                if DI.SetRotateAngle(-1.0*rotAng):
                    if DI.SetIncidentGravityParam( B ):
                        out_file = os.path.join(os.getcwd(), "new_"+os.path.basename(dfile_org))
                        if DI.Write(out_file):
                            print("CreateRPMTDetectorInfoFromData >>> Output File = {}".format(out_file))
                            # DI.Dump()
                        else:
                            print("Failed to save as {}".format(out_file))
                            return False
                else:
                    print("Failed to SetRotateAngle with {} [deg]".format(rotAng))
                    return False
            else:
                print("Failed to SetBeamCenterPixel with {}, {} [pixel]".format(drop_origin_px, drop_origin_py))
                return False
        else:
            return False
    else:
        print("Failed to Read {}".format(dfile_org))
        return False

    # Data with final detector info
    DAT_res = GetHistogramSAS( runNo=runNo, Xrange="0.7 15.5 0.1 dL", Bank="RPMT", TimeSlice="-1 -1", frameBoundary="0.0", isTimeFoc=False, SolAngCor=True, paramFiles="- " + out_file )
    DAT_res.MulMySelf(factor)

    if withPlot:
        import VisualModule as VB
        p1 = VB.MPlot([EC, EC2, EC3])
        p1.SetXScale(27.0, 35.0)
        p1.SetYScale(24.0, 32.0)
        p1.SetPlotParam(1, ls="", mk="o", eb=False)
        p1.SetPlotParam(2, ls="-", mk="", eb=False)
        p1.SetPlotParam(3, ls="", mk="s", eb=False)
        p2 = VB.MPlot([EC4, EC5])
        p2.SetYScale(-6.5, 0.5)
        p2.SetPlotParam(1, ls="", mk="o", eb=False)
        p2.SetPlotParam(2, ls="-", mk="", eb=False)
        p2.SetXLabel("Lambda")
        p2.SetYLabel("Drop distance")
        VB.M2PlotPlus(DAT_res)

    return DAT_res

#########################################
def GetSumPixelArea( dat, PixelRange ):
    p_list_s = PixelRange.split(" ")
    p_list = []
    for a_posi in p_list_s:
        if a_posi!="":
            p_list.append( int(a_posi) )
    if len(p_list)!=4:
        raise UserWarning("PixelRange is invalid.( must be 4 values )")
    x0 = p_list[0]
    y0 = p_list[1]
    x1 = p_list[2]
    y1 = p_list[3]
    if x0>=(dat.PutSize()) or x1>=(dat.PutSize()) or y0>=(dat(0).PutSize()) or y1>=(dat(0).PutSize()) or x0<0 or y0<0 or x1<0 or y1<0:
        raise UserWarning("PixelRange is invalid. (%d,%d,%d,%d)"%(tuple(p_list)))

    det_v = Manyo.MakeUInt4Vector()
    pix_v = Manyo.MakeUInt4Vector()
    for px in range( x0, x1+1 ):
        for py in range( y0, y1+1 ):
            det_v.append(px)
            pix_v.append(py)

    import Manyo.Utsusemi as mu
    tt=mm.AverageElementContainerMatrix(dat)
    tt.SetPoints( det_v, pix_v )
    EC = tt.GetSum()
    del tt

    EC.InputHeader(dat.PutHeader())
    lamb_vect = dat(x0,y0).Put("Lamb")
    EC.Add("Lamb",lamb_vect)
    return EC

#########################################
def GetHistInelaSAS( runNo="0", Bank="SM", Ei=10.0, Erange="-5 5", dHW=0.1):
    """
    Histogram Creation for inelastic scattering on SAS

    @param runNo     (string)  Run Number "10,11,12"
    @param Bank      (string)  "SM"...  Monitors by "MON1" or "MON2"
    @param Ei        (float)   Incident energy
    @param Erange    (string)  <hw_min>, <hw_max>
    @param dHW       (float)   bin width of hw
    @retval DAT (ElementContainerMatrix)
    """
    UGH = mu.UtsusemiGetNeunetHistogram()
    if not UGH.SetRunNo( runNo ):
        raise UserWarning( "RunNo is invalid." )

    if len(Erange.split(" "))>=len(Erange.split(",")):
        hw_range_st = Erange.split(" ")
    else:
        hw_range_st = Erange.split(",")
    if len(hw_range_st)>=2:
        hw_min = float(hw_range_st[0])
        hw_max = float(hw_range_st[1])
        if hw_max >= Ei:
            raise UserWarning( "Erange hw_max is too large (<Ei)." )
    else:
        raise UserWarning( "Erange is invalid." )

    UUC = mu.UtsusemiUnitConverter()
    print("Incident Lambda = {}".format( UUC.EtoLambda(Ei) ))
    print("Lambda (Ei - hw_max) = {}".format( UUC.EtoLambda(Ei-hw_max) ))
    print("Lambda (Ei - hw_min) = {}".format( UUC.EtoLambda(Ei-hw_min) ))
    params = "hw,{},{},{},{}".format( Ei, hw_min, hw_max, dHW )
    if not UGH.SetConversionParameter( params ):
        raise UserWarning( "Params is invalid.({})".format(params) )

    if not UGH.SetDetParam("psd"):
        raise UserWarning( "Det Type is invalid.({})".format("psd") )


    if len(Bank.split(","))>=len(Bank.split(" ")):
        bank_list = Bank.split(",")
    else:
        bank_list = Bank.split(" ")
    Bank_fixed = ""
    for a_bank in bank_list:
        if a_bank=="SM":
            Bank_fixed += "SMBL14,SMBL16,"
        else:
            Bank_fixed += (a_bank+",")
    Bank_fixed = Bank_fixed[:-1]
    print("Bank_fixed = ",Bank_fixed)
    if not UGH.SetDetRange( Bank_fixed ):
        raise UserWarning( "Bank is invalid.({})".format(Bank))

    UGH.SetFlagToAddTofBin(True)
    wfile = UGH.MakeTempWiringInfo()
    dfile = UGH.MakeTempDetectorInfo()
    #del UGH
    runNos = runNo.split(",")
    runNov = Manyo.MakeUInt4Vector()
    for a_run in runNos:
        runNov.push_back( int(a_run) )
    DAT = Manyo.ElementContainerMatrix()
    UGH.SetElementContainerMatrix(DAT,runNov,wfile,dfile,False,"/data","")
    del UGH


    """
    tt=mu.UtsusemiEventDataConverterNeunet()
    tt.LoadParamFiles( wfile, dfile )
    tt.SetHistAllocation()
    DAT = Manyo.ElementContainerMatrix()
    tt.SetElementContainerMatrix( DAT, runNo, "/data","" )
    del tt
    """

    pp=mu.UtsusemiReductionInEla(DAT)
    pp.CalcPhi()
    pp.KiKfCorrect()
    pp.dHWCorrect()
    #pp.NormBySolidAngle()
    del pp

    return DAT

#########################################



# Writing and reading NeXus file
# This sample script will create a file, "test.nx".

import Manyo

# Preparing Manyo-Lib data container 
e1 = Manyo.ElementContainer()
vec = Manyo.MakeDoubleVector( 100 )
for i in range( 100 ):
    e1.Add( "x"+str(i), vec )
# Preparing data container finished


# Making the class object of the wrapper of file I/O. 
ne1 = Manyo.NeXusFileIO()

# ( data_object_of_Manyo_library, file_name, User_name )
ne1.Write( e1, "test.nx", "suzuki" )

# close data file
del ne1

###### writing NeXus file finished

# Making the class object of the wrapper of file I/O.
ne2 = Manyo.NeXusFileIO()

# reading data file with the file name
e2 = ne2.ReadElementContainer( "test.nx" )

# close data file
del ne2

###### reading NeXus file finished

e2.Dump()





#!/usr/bin/python

import Manyo
import BuildInFunc

paramForGaussian=Manyo.DoubleVector();
paramForGaussian.push_back(1.0)  # height
paramForGaussian.push_back(0.0)  # cneter
paramForGaussian.push_back(1.0)  # width

xmin=-5.0
xmax=5.0
div=100
delta=(xmax-xmin)/div

f=BuildInFunc.Lorentzian()
for i in range(div+1):
    x=xmin+delta*i
    g= f.gradient(x, paramForGaussian)
    print x, f.eval(x, paramForGaussian), f.der1st(x, paramForGaussian), f.der2nd(x, paramForGaussian), g[0], g[1], g[2]
    



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function
import time
import os
import Manyo
import Manyo.Utsusemi as mu
import math
from utsusemi.ana.Reduction.BaseCommands import *

#########################################
# BaseCommands for Elastic
#########################################


def GetHistDTconstNeunetPsd(runNo=0, isTimeFoc=True, tofRange="0.0 40000.0", binTOF=10.0, detRange="-1 -1", NumOfPixel=100, timeRange="-1 -1", frameBoundary=0.0):
    """
    Load EventData produced Neunet PSD and convert to histograms with deltaT const.
    GetHistDTconstNeunetPsd( runNo=0,isTimeFoc=True,tofRange="0.0 40000.0",binTOF=10.0,
                             detRange="-1 -1",NumOfPixel=Def_NumOfPixel,
                             timeRange="-1 -1",frameBoundary=0.0 )

    @param runNo     (int) RunNumber
    @param isTimeFoc (bool) True: use Time Focusing
    @param tofRange  (string) startTOF,endTOF
    @param binTOF    (int)    bin width
    @param detRange  (string) first Detector, last Detector
    @param NumOfPixel (int) number of pixels on a detector
    @param timeRange (string) start time, end time [seconds]
    @param frameBoundary (int)
    @retval ElementContainerMatrix
    """
    if isinstance(isTimeFoc, bool):
        raise UserWarning("isTimeFoc is unknown.")
    ConvType = 2
    if isTimeFoc:
        ConvType = 12
    tof_range = SplitStringParameters(tofRange, 2, "float")
    if isinstance(tof_range, int) and tof_range < 0:
        raise UserWarning("tofRange is invalid.")
    det_range = SplitStringParameters(detRange, 2, "int")
    if isinstance(det_range, int) and det_range < 0:
        raise UserWarning("detRange is invalid.")
    time_range = SplitStringParameters(timeRange, 2, "float")
    if isinstance(time_range, int) and time_range < 0:
        raise UserWarning("timeRange is invalid.")

    fInfo = [0, 0.0]
    if frameBoundary != 0.0:
        if frameBoundary <= 40000.0:
            fInfo[0] = 2
            fInfo[1] = frameBoundary
        elif frameBoundary <= 80000.0:
            fInfo[0] = 3
            fInfo[1] = frameBoundary - 40000.0
        else:
            raise UserWarning("Invalid frameBoundary.")

    runNo = str(runNo)
    key_hist = ""
    if isTimeFoc:
        key_hist = "tf-tof"
    else:
        key_hist = "tof"
    HistParam = "%s,%f,%f,%f" % (key_hist, tof_range[0], tof_range[1], binTOF)
    DetParam = "psd"
    DetRange = ""
    if det_range[0] == -1 and det_range[1] == -1:
        DetRange = "All"
    else:
        DetRange = "%d-%d" % (det_range[0], det_range[1])
    TimeRange = "%f,%f" % (time_range[0], time_range[1])
    FrameInfo = "%d,%f" % (fInfo[0], fInfo[1])

    return GetNeunetHist(runNo=runNo,
                         HistParam=HistParam,
                         DetParam=DetParam,
                         DetRange=DetRange,
                         TimeRange=TimeRange,
                         FrameInfo=FrameInfo)

#########################################


def GetHistToDTconstNeunetPsd(runNo=0, isTimeFoc=True, tofRange="0.0 40000.0", racio=0.01, detRange="-1 -1", NumOfPixel=100, timeRange="-1 -1", frameBoundary=0.0):
    """
    Load EventData produced Neunet PSD and convert to histograms with T over deltaT const.
    GetHistToDTconstNeunetPsd( runNo=0,isTimeFoc=True,tofRange="0.0 40000.0",racio=0.01,
                               detRange="-1 -1",NumOfPixel=Def_NumOfPixel,
                               timeRange="-1 -1",frameBoundary=0.0 )

    @param runNo     (int) RunNumber
    @param isTimeFoc (bool) True: use Time Focusing
    @param tofRange  (string) startTOF,endTOF
    @param racio     (float)  dT/T const value
    @param detRange  (string) first Detector, last Detector
    @param NumOfPixel (int) number of pixels on a detector
    @param timeRange (string) start time, end time [seconds]
    @param frameBoundary (int)
    @retval ElementContainerMatrix
    """
    if isinstance(isTimeFoc, bool):
        raise UserWarning("isTimeFoc is unknown.")
    ConvType = 3
    if isTimeFoc:
        ConvType = 13
    tof_range = SplitStringParameters(tofRange, 2, "float")
    if isinstance(tof_range, int) and tof_range < 0:
        raise UserWarning("tofRange is invalid.")
    det_range = SplitStringParameters(detRange, 2, "int")
    if isinstance(det_range, int) and det_range < 0:
        raise UserWarning("detRange is invalid.")
    time_range = SplitStringParameters(timeRange, 2, "float")
    if isinstance(time_range, int) and time_range < 0:
        raise UserWarning("timeRange is invalid.")

    fInfo = [0, 0.0]
    if frameBoundary != 0.0:
        if frameBoundary <= 40000.0:
            fInfo[0] = 2
            fInfo[1] = frameBoundary
        elif frameBoundary <= 80000.0:
            fInfo[0] = 3
            fInfo[1] = frameBoundary - 40000.0
        else:
            raise UserWarning("Invalid frameBoundary.")

    runNo = str(runNo)
    key_hist = ""
    if isTimeFoc:
        key_hist = "tf-rtof"
    else:
        key_hist = "rtof"
    HistParam = "%s,%f,%f,%f" % (key_hist, tof_range[0], tof_range[1], racio)
    DetParam = "psd"
    DetRange = ""
    if det_range[0] == -1 and det_range[1] == -1:
        DetRange = "All"
    else:
        DetRange = "%d-%d" % (det_range[0], det_range[1])
    TimeRange = "%f,%f" % (time_range[0], time_range[1])
    FrameInfo = "%d,%f" % (fInfo[0], fInfo[1])

    return GetNeunetHist(runNo=runNo,
                         HistParam=HistParam,
                         DetParam=DetParam,
                         DetRange=DetRange,
                         TimeRange=TimeRange,
                         FrameInfo=FrameInfo)


#########################################
def GetHistDQconstNeunetPsd(runNo=0, Qrange="0.1 10.0", deltaQ=0.1, detRange="-1 -1", NumOfPixel=100, timeRange="-1 -1", frameBoundary=0.0, inVect="0.0 0.0 1.0"):
    """
    Load EventData produced Neunet PSD and convert to histograms with deltaQ const.
    GetHistDQconstNeunetPsd( runNo=0,Qrange="0.1 10.0",deltaQ=0.1,
                             detRange="-1 -1",NumOfPixel=Def_NumOfPixel,
                             timeRange="-1 -1",frameBoundary=0.0, inVect="0.0 0.0 1.0" )

    @param runNo     (int) RunNumber
    @param QRange  (string) Q min , Q max
    @param deltaQ  (int)    delta Q
    @param detRange  (string) first Detector, last Detector
    @param NumOfPixel (int) number of pixels on a detector
    @param timeRange (string) start time, end time [seconds]
    @param frameBoundary (int)
    @param inVect  (string) incident beam vector "vx,vy,vz"
    @retval ElementContainerMatrix
    """
    Q_range = SplitStringParameters(Qrange, 2, "float")
    if isinstance(Q_range, int) and Q_range < 0:
        raise UserWarning("Q Range is invalid.")
    det_range = SplitStringParameters(detRange, 2, "int")
    if isinstance(det_range, int) and det_range < 0:
        raise UserWarning("detRange is invalid.")
    time_range = SplitStringParameters(timeRange, 2, "float")
    if isinstance(time_range, int) and time_range < 0:
        raise UserWarning("timeRange is invalid.")
    in_vect = SplitStringParameters(inVect, 3, "float")
    if isinstance(in_vect, int) and in_vect < 0:
        raise UserWarning("in vect is invalid.")

    fInfo = [0, 0.0]
    if frameBoundary != 0.0:
        if frameBoundary <= 40000.0:
            fInfo[0] = 2
            fInfo[1] = frameBoundary
        elif frameBoundary <= 80000.0:
            fInfo[0] = 3
            fInfo[1] = frameBoundary - 40000.0
        else:
            raise UserWarning("Invalid frameBoundary.")

    params = [Q_range[0], Q_range[1], deltaQ,
              in_vect[0], in_vect[1], in_vect[2]]

    runNo = str(runNo)
    HistParam = "q,%f,%f,%f,%f,%f,%f" % (
        Q_range[0], Q_range[1], deltaQ, in_vect[0], in_vect[1], in_vect[2])
    DetParam = "psd"
    DetRange = ""
    if det_range[0] == -1 and det_range[1] == -1:
        DetRange = "All"
    else:
        DetRange = "%d-%d" % (det_range[0], det_range[1])
    TimeRange = "%f,%f" % (time_range[0], time_range[1])
    FrameInfo = "%d,%f" % (fInfo[0], fInfo[1])

    return GetNeunetHist(runNo=runNo,
                         HistParam=HistParam,
                         DetParam=DetParam,
                         DetRange=DetRange,
                         TimeRange=TimeRange,
                         FrameInfo=FrameInfo)

#########################################


def GetHistDTconstGatenetMon(runNo=0, tofRange="0.0 40000.0", binTOF=10.0, detRange="-1 -1", NumOfPixel=4, timeRange="-1 -1", frameBoundary=0.0):
    """
    Load EventData produced GateNET monitor and convert to histograms with deltaT const.
    GetHistDTconstGatenetMon( runNo=0,tofRange="0.0 40000.0",binTOF=10.0,
                              detRange="-1 -1",NumOfPixel=4,
                              timeRange="-1 -1",frameBoundary=0.0 )

    @param runNo     (int) RunNumber
    @param tofRange  (string) startTOF,endTOF
    @param binTOF    (int)    bin width
    @param detRange  (string) first Detector, last Detector
    @param NumOfPixel (int) number of pixels on a detector
    @param timeRange (string) start time, end time [seconds]
    @param frameBoundary (int)
    @retval ElementContainerMatrix
    """

    tof_range = SplitStringParameters(tofRange, 2, "float")
    if isinstance(tof_range, int) and tof_range < 0:
        raise UserWarning("tofRange is invalid.")
    det_range = SplitStringParameters(detRange, 2, "int")
    if isinstance(det_range, int) and det_range < 0:
        raise UserWarning("detRange is invalid.")
    time_range = SplitStringParameters(timeRange, 2, "float")
    if isinstance(time_range, int) and time_range < 0:
        raise UserWarning("timeRange is invalid.")

    fInfo = [0, 0.0]
    if frameBoundary != 0.0:
        if frameBoundary <= 40000.0:
            fInfo[0] = 2
            fInfo[1] = frameBoundary
        elif frameBoundary <= 80000.0:
            fInfo[0] = 3
            fInfo[1] = frameBoundary - 40000.0
        else:
            raise UserWarning("Invalid frameBoundary.")

    runNo = str(runNo)
    HistParam = "tof,%f,%f,%f" % (tof_range[0], tof_range[1], binTOF)
    DetParam = "n2mon"
    DetRange = ""
    if det_range[0] == -1 and det_range[1] == -1:
        DetRange = "All"
    else:
        DetRange = "%d-%d" % (det_range[0], det_range[1])
    TimeRange = "%f,%f" % (time_range[0], time_range[1])
    FrameInfo = "%d,%f" % (fInfo[0], fInfo[1])

    return GetNeunetHist(runNo=runNo,
                         HistParam=HistParam,
                         DetParam=DetParam,
                         DetRange=DetRange,
                         TimeRange=TimeRange,
                         FrameInfo=FrameInfo)

#########################################


def GetHistToDTconstGatenetMon(runNo=0, tofRange="0.0 40000.0", racio=0.01, detRange="-1 -1", NumOfPixel=4, timeRange="-1 -1", frameBoundary=0.0):
    """
    Load EventData produced GateNET monitor and convert to histograms with T over deltaT const.
    GetHistToDTconstGatenetMon( runNo=0,tofRange="0.0 40000.0",racio=0.01,
                                detRange="-1 -1",NumOfPixel=4,
                                timeRange="-1 -1",frameBoundary=0.0 )

    @param runNo     (int) RunNumber
    @param tofRange  (string) startTOF,endTOF
    @param racio     (float)  dT/T const
    @param detRange  (string) first Detector, last Detector
    @param NumOfPixel (int) number of pixels on a detector
    @param timeRange (string) start time, end time [seconds]
    @param frameBoundary (int)
    @retval ElementContainerMatrix
    """

    tof_range = SplitStringParameters(tofRange, 2, "float")
    if isinstance(tof_range, int) and tof_range < 0:
        raise UserWarning("tofRange is invalid.")
    det_range = SplitStringParameters(detRange, 2, "int")
    if isinstance(det_range, int) and det_range < 0:
        raise UserWarning("detRange is invalid.")
    time_range = SplitStringParameters(timeRange, 2, "float")
    if isinstance(time_range, int) and time_range < 0:
        raise UserWarning("timeRange is invalid.")

    fInfo = [0, 0.0]
    if frameBoundary != 0.0:
        if frameBoundary <= 40000.0:
            fInfo[0] = 2
            fInfo[1] = frameBoundary
        elif frameBoundary <= 80000.0:
            fInfo[0] = 3
            fInfo[1] = frameBoundary - 40000.0
        else:
            raise UserWarning("Invalid frameBoundary.")

    runNo = str(runNo)
    HistParam = "rtof,%f,%f,%f" % (tof_range[0], tof_range[1], racio)
    DetParam = "n2mon"
    DetRange = ""
    if det_range[0] == -1 and det_range[1] == -1:
        DetRange = "All"
    else:
        DetRange = "%d-%d" % (det_range[0], det_range[1])
    TimeRange = "%f,%f" % (time_range[0], time_range[1])
    FrameInfo = "%d,%f" % (fInfo[0], fInfo[1])

    return GetNeunetHist(runNo=runNo,
                         HistParam=HistParam,
                         DetParam=DetParam,
                         DetRange=DetRange,
                         TimeRange=TimeRange,
                         FrameInfo=FrameInfo)
#########################################


def SaveDataToNVASQ(dat, path="./", filename="Sample.nx", L1=25000.0, px=15000.0, py=0.0, pz=0.0):
    """
    Save ElementContainer to NeXus file with nvaSp.py format.(for BL11 only)
    @param  dat      (ElementContainer)
    @param  path     (String) path to data file
    @param  filename (String) data file name
    @param  L1 (Double)
    @param  px (Double) x position of a pixel
    @param  py (Double) y position of a pixel
    @param  pz (Double) z position of a pixel
    @retval None
    """
    if not isinstance(dat, Manyo.ElementContainer):
        raise UserWarning(" dat argument must be ElementContainer.")

    ec = Manyo.ElementContainer()
    ec.Add("TOF", dat.PutX(), dat.PutUnit(ec.PutXKey()))
    ec.Add("Intensity", dat.PutY(), dat.PutUnit(ec.PutYKey()))
    ec.Add("Error", dat.PutE(), dat.PutUnit(ec.PutEKey()))
    ec.SetKeys("TOF", "Intensity", "Error")
    eca = Manyo.ElementContainerArray()
    eca.Add(ec)

    hh = eca.PutHeaderPointer()
    hh.Add("L1", L1)

    pv = Manyo.MakeDoubleVector()
    pv.append(float(px))
    pv.append(float(py))
    pv.append(float(pz))

    hh1 = eca(0).PutHeaderPointer()
    hh1.Add("PixelPosition", pv)
    # hh1.Add("Lsintheta",17677.67)
    hh1.Add("Lsintheta", (L1 + 1500.0) * 0.707106781)
    hh1.Add("ttheta", 1.5708)
    hh1.Add("L2", 1500.0)

    nexusIO = Manyo.NeXusFileIO()
    nexusIO.Write(eca, os.path.join(path, filename), "BL11", 0)
    del nexusIO



__AnaFacades__ = ["DR", "Cmm", "Com"]
__VisFacades__ = ["D1Plotter"]
"""
__Structures__ = [("File",["Cmm.SaveDataToManyoBinary",
                           "Cmm.LoadDataFromManyoBinary",
                           "Cmm.SaveDataToDAVEgrp",
                           "Cmm.SaveDataToDump",
                           "Cmm.LoadDataFromDump"]),
                  ("DataReduct",["Cmm.GetNeunetHist",
                                 "Cmm.GetMultiEi2",
                                 "DR.GetDataOfMonochroEi2",
                                 "DR.GetDataOfMonochroEi3",
                                 "DR.ToPowder",
                                 "DR.MakeRunList"])
                  ]
"""



from Manyo import HeaderBase, ElementContainer, ElementContainerArray, ElementContainerMatrix, Int4Vector,\
                StringVector, DoubleVector ##, DoubleVectorContainer

from Manyo import SearchInHeader

from Manyo import ListToDoubleVector as l2d
import os, sys, string
import numpy

from sub1 import simplehistogram, ecsub, eccopy


def ChopperMaskAuto(ecm, lowerlimit = 1.0e-4, upperlimit = 1.0e4, wrinteonfile = True, filename = None):
    """
    This function make masking file automatically.
    At first, it calculate the sum of each ElementContainer and find their medium(not average) value.
    Then if the sum of intensity is less than lowerlimit x medium or more than upperlimit x medium, 
    the index of the ElementContainer in ElementContainerMatrix is stored.
    Finally, the indexes can be stored in file if @writeonfile == True. If masking file is not appointed as function arguement,
    the default filname is "YYYYMMDD_HHMMSS.msk". (year, month, day, hour, minute, second)
    If @writeonfile is not True, it returns its masking table.
    """
    
    
    flux={}
    tableindex=[]
    maskedindex=[]
    ecanumber = ecm.PutTableSize()
    for ecaindex in range(ecanumber):
        ecnumber = ecm(ecaindex).PutTableSize() 
        for ecindex in range(ecnumber):
            tempec=ecm(ecindex, ecaindex)
            #tableindex.append((ecaindex, ecindex))
            flux[(ecaindex, ecindex)]=tempec.Sum(tempec.PutYKey())
    flux0 = numpy.array(flux.values(), dtype='float')
    flux0.sort()
    flux_medium = flux0[len(flux0)/2-1]
    #tableindex = tuple(tableindex)
    for index0 in flux.keys():
        if flux[index0] <= lowerlimit*flux_medium or flux[index0] >= upperlimit*flux_medium :
            maskedindex.append(index0)
    
    maskedindex.sort()    
    if writeonfile == True :
        if filename == None :
            import time
            tm = time.localtime()
            fn = str("%04d%02d%02d_%02d%02d%02d.msk"%(tm[0],tm[1],tm[2],tm[3],tm[4],tm[5]))
        else :
            fn=filename
        mfile = open(fn, "w")
        for index0 in maskedindex :
            mfile.write("%s.%s\n"%(index0[0],index0[1]))
        mfile.close()
        #return
        return None # added by inamura (2008.09.03)
    else :
        return maskedindex
    
def ChopperMask(ecm, masked=None, maskingfile=None):
    """
    This function at first read masked index as list of index or file.
    Then, it write "MASKED" key as 1 in HeaderBase in each ElementContainer
    
    """
    
    maskindex = []
    if maskingfile != None :
        mkfile = open(maskingfile, "r")
        lines = mkfile.readlines()
        for line in lines :
            ll = line.split()
            if len(ll) == 0:
                pass
            elif ll[0] is "#" :
                pass
            else:
                for ls in ll:
                    qs = ls.split(".")
                    if len(qs) == 1 or (len(qs)==2 and len(qs[1])==0) :
                        maskindex.append((int(qs[0]),))
                    elif len(qs) == 2:
                        maskindex.append((int(qs[0]), int(qs[1])))
        maskindex = tuple(maskindex)
    elif masked != None :
        maskindex = masked    

    #if type(ecm) != type(ElementContainerMatrix()):
    if not isinstance( ecm, ElementContainerMatrix ):
        raise AttributeError, "Input must be Manyo.ElementContainerMatrix"
    if type(maskindex) not in [type([1,1]), type((1,))]:
        raise AttributeError, "Parameters appointing detector to be masked must be Python.List or Python.Tuple"
    ## [Inamura 091222,130702]-->
    HIS = SearchInHeader(ecm)
    ## <--[Inamura 091222,130702]
    for mm in maskindex :
        ## [Inamura 091222,130702]-->
        #print mm
        HIS.SearchArray("PSDID",mm[0])
        res = HIS.PutResultIndex(0)
        if res.size()==0:
            print "PSD-ID:",mm[0]," is not found."
            continue
        if len(mm) == 1:
            mm = (res[0],)
        elif len(mm) == 2:
            mm = (res[0],mm[1])
        else:
            continue
        ##<--[Inamura 091222,130702]
        if len(mm) == 1:
            ecm(mm[0]).PutHeaderPointer().OverWrite("MASKED",1)
            for jj in range(ecm(mm[0]).PutTableSize()):
                tempheader = ecm(mm[0], jj).PutHeader()
                tempheader.OverWrite("MASKED",1)
                ecm(mm[0],jj).InputHeader(tempheader)
        elif len(mm) == 2:
            tempheader = ecm(mm[0], mm[1]).PutHeader()
            tempheader.OverWrite("MASKED",1)
            ecm(mm[0],mm[1]).InputHeader(tempheader)
            echeader = ecm(mm[0]).PutHeaderPointer()
            vind = echeader.PutInt4Vector("MASKEDElementContainer")
            vind.append(mm[1])
            echeader.OverWrite("MASKEDElementContainer", vind)
    ecmhd = ecm.PutHeaderPointer()
    ecmhdprc = ecmhd.PutStringVector("DATAPROCESSED")
    ecmhdprc.append("MASK")
    ecmhd.OverWrite("DATAPROCESSED", ecmhdprc)
    ## [Inamura 091222]-->
    del HIS
    ## <--[Inamura 091222]
    return ecm
    


def BackgroundSubtraction(ecm, method, info): # change ec->ecm by inamura (2008.09.03)
    
    """
    Background Subtraction Method
    
    @  method  
        0 : Constant Subtraction for All histogram 
        1 : Average of Both End Points
        2 : Average of Both End range
        3 : Background is given by List or Array
    @ info 
        if method is 0 : Constant 
                     1 : (Number of Point at starting, Number of point at ending) 
                     2 : (range of left end point, right end point)
                     3 : BG array
    Notice >
        if method 1 or 2 and length of info is 1, then it is used for both.
        
        if BG array is 1-dimensional and the size is same as ElementContainerArray number
            in ElementContainerMatrix, one element in BG array is considered as BG for 
            ElementContainerArray  
        
    """
    BGSubMethod = { "By Constant" : 0,              
                    "By Average of End Point" : 1,
                    "By Average of End Range": 2,
                    "By Array" : 3 }
    
    #ecm = eccopy(ec)  removed by inamura (2008.09.03)

    if type(ecm) == type(None):
        raise AttributeError, "Input must be Manyo.ElementContainerMatrix"
    BGS_Method = 0
    
    
    if type(method) == type(1) and method in BGSubMethod.values():
        BGS_Method = method
    elif type(method) == type("a") and method in BGSubMethod.keys():
        BGS_Method = BGSubMethod(method)
    else :
        raise AttributeError, "Background Subtraction Method is not specified"
    
    if BGS_Method == 0:
        if type(info) not in (type(1), type(1.0)):
            raise ValueError, "Must be value" 
        #if type(ecm) is type(ElementContainer()):
        if isinstance( ecm, ElementContainer ):
            ecsub(ecm, info)
        #elif type(ecm) is type(ElementContainerArray()):
        elif isinstance( ecm, ElementContainerArray ):
            for ind in range(ecm.PutTableSize()):
                ec=ecm.PutPointer(ind)
                ecsub(ec, info)
        #elif type(ecm) is type(ElementContainerMatrix()):
        elif isinstance( ecm, ElementContainerMatrix ):
            for ind1 in range(ecm.PutTableSize()):
                eca = ecm.PutPointer(ind1)
                print "---[eca table size=",ind1,eca.PutTableSize()
                for ind2 in range(eca.PutTableSize()):
                    ec = eca.PutPointer(ind2)
                    ecsub(ec, info)
    
    elif BGS_Method == 1 :
        if type(info) in [type(1), type(1.0)]:
            nn = numpy.array([info])
        else:
            nn = numpy.array(info)
        
        if len(info) == 1:
            nl = nr = nn[0]
        elif len(info) == 2:
            nl = nn[0]
            nr = nn[1]
        else :
            raise ValueError, "Must be lentgh 2 integer sequence - list, tuple, array" 
                
        #if type(ecm) is type(ElementContainer()):
        if isinstance( ecm, ElementContainer ):
            yd = numpy.array(ecm.PutYList())
            bg = (yd[:nl].sum()+yd[-nr:].sum())/float(nl+nr)
            ecsub(ecm, bg)
        #elif type(ecm) is type(ElementContainerArray()):
        elif not isinstance( ecm, ElementContainerArray ):
            for ind in range(ecm.PutTableSize()):
                ec=ecm.PutPointer(ind)
                yd = numpy.array(ec.PutYList())
                bg = (yd[:nl].sum()+yd[-nr:].sum())/float(nl+nr)
                ecsub(ec, bg)
        #elif type(ecm) is type(ElementContainerMatrix()):
        elif isinstance( ecm, ElementContainerMatrix ):   
            for ind1 in range(ecm.PutTableSize()):
                eca = ecm.PutPointer(ind1)
                for ind2 in range(eca.PutTableSize()):
                    ec = eca.PutPointer(ind2)
                    yd = numpy.array(ec.PutYList())
                    bg = (yd[:nl].sum()+yd[-nr:].sum())/float(nl+nr)
                    ecsub(ec, bg)
        else :
            raise ValueError, "Wrong Background Number"
        
        
    elif BGS_Method == 2:
        if type(info) in [type(1), type(1.0)]:
            nn = numpy.array([info])
        else:
            nn = numpy.array(info)
        
        if len(nn) == 1:
            nl = nr = float(nn[0])
        elif len(info) == 2:
            nl = float(nn[0])
            nr = float(nn[1])
        else :
            raise ValueError, "Must be lentgh 2 integer sequence - list, tuple, array" 
                
        #if type(ecm) is type(ElementContainer()):
        if isinstance( ecm, ElementContainer ):
            xd0 = numpy.array(ecm.PutXList())
            xd = (xd0[1:]+xd0[:-1])*0.5
            yd = numpy.array(ecm.PutYList())
            bgr = []
            for ind in range(len(xd)):
                if abs(xd[ind]-xd0[0]) <= nl or abs(xd0[-1]-xd[ind]) <= nr : 
                    bgr.append(yd[ind])
            bgr = numpy.array(bgr)
            bg = bgr.sum()/len(bgr)
            ecsub(ecm, bg)
        #elif type(ecm) is type(ElementContainerArray()):
        elif not isinstance( ecm, ElementContainerArray ):
            for ind in range(ecm.PutTableSize()):
                ec=ecm.PutPointer(ind)
                xd0 = numpy.array(ec.PutXList())
                xd = (xd0[1:]+xd0[:-1])*0.5
                yd = numpy.array(ec.PutYList())
                bgr = []
                for ind in range(len(xd)):
                    if abs(xd[ind]-xd0[0]) <= nl or abs(xd0[-1]-xd[ind]) <= nr : 
                        bgr.append(yd[ind])
                bgr = numpy.array(bgr)
                bg = bgr.sum()/len(bgr)
                ecsub(ec, bg)
        #elif type(ecm) is type(ElementContainerMatrix()):
        elif isinstance( ecm, ElementContainerMatrix ):
            for ind1 in range(ecm.PutTableSize()):
                eca = ecm.PutPointer(ind1)
                for ind2 in range(eca.PutTableSize()):
                    ec = eca.PutPointer(ind2)
                    xd0 = numpy.array(ec.PutX())
                    xd = (xd0[1:]+xd0[:-1])*0.5
                    yd = numpy.array(ec.PutY())
                    bgr = []
                    for ind in range(len(xd)):
                        if abs(xd[ind]-xd0[0]) <= nl or abs(xd0[-1]-xd[ind]) <= nr : 
                            bgr.append(yd[ind])
                    bgr = numpy.array(bgr)
                    bg = bgr.sum()/len(bgr)
                    ecsub(ec, bg)
        
        else :
            raise ValueError , "Wrong Background Range"
        
    elif BGS_Method == 3:
        BGarr = info
            
        #if type(ecm) == type(ElementContainerArray()):
        if isinstance( ecm, ElementContainerArray ):
            if len(BGarr) != ecm.PutTableSize():
                raise ValueError, "Wrong Backgroud Array size"
            for ind in range(ecm.PutTableSize()):
                ec=ecm.PutPointer(ind)
                ecsub(ec, BGarr[ind])
        #elif type(ecm) == type(ElementContainerMatrix()) :
        elif isinstance( ecm, ElementContainerMatrix ):
            if len(BGarr) != ecm.PutTableSize():
                raise ValueError, "Wrong Backgroud Array size"
            for ind1 in range(ecm.PutTableSize()):
                eca = ecm.PutPointer(ind1)
                for ind2 in range(eca.PutTableSize()):
                    ec = eca.PutPointer(ind2)
                    if type(BGarr[ind1]) in (type(1.0), type(1)):
                        ecsub(ec, BGarr[ind1])
                    elif type(BGarr[ind1]) in (type([1,]), type((1,)), type(numpy.array([1,]))):
                        if len(BGarr[ind1]) == eca.PutTableSize():
                            ecsub(ec, BGarr[ind1, ind2])
                        else :
                            raise ValueError, "Wrong Backgroud Array size"
                    else :
                        raise ValueError, "Wrong Backgroud Array size"
    else :
        raise ValueError, "Wrong Backgroud Method or Information"
    return ecm
    

#def Tof2Energy(ecm, L2p, Lms, t_offset):
def Tof2Energy(ecm, Lms, t_offset):        #changed by inamura (2008.08.04)
    """
    Function to convert Time-of-Flight to Energy 
    
    Arguements:
        ecm : Elememt Container Matrix
        Ei (meV) : Incident Neutron Energy
        L2p (m) : list of sample and detector pixel distance
        Lms (m) : monitor to sample distance
        t_offset (microsec) : time-of-flight shift. 
    """
    #Ei = ecm.PutHeader().PutDouble("IncidentNeutronEnergy")
    Ei = ecm.PutHeader().PutDouble("Ei") # changed by inamura (2008.09.04)
    import numpy
    if ecm(0).PutHeaderPointer().PutString("TYPE")=="MONITOR":
        start_ind0 = 1
    else:
        start_ind0 = 0
    for ind0 in range(start_ind0,ecm.PutTableSize()):
        print "PSD=",ind0
        for ind1 in range(ecm(ind0).PutTableSize()):
            xkey = ecm(ind0, ind1).PutXKey()
            ykey = ecm(ind0, ind1).PutYKey()
            ekey = ecm(ind0, ind1).PutEKey()
            tof = numpy.array(ecm(ind0, ind1).PutX()) - t_offset
            vi = 437.4 * numpy.sqrt(Ei)
            mn = 1.6749e-27  # Neutron mass
            J2meV = 6.24151e21 # Joule to meV
            '''
            if type(Lms) == type([]):
                energy = Ei - 0.5 * mn * (L2p*L2p)/(tof*1.0e-6 - Lms[ind0][ind1]/vi)**2
            else :
                energy = Ei - 0.5 * mn * (L2p*L2p)/(tof*1.0e-6 - Lms/vi)**2 * J2meV
            '''
            PosiVect = ecm(ind0, ind1).PutHeaderPointer().PutDoubleVector("PixelPosition")
            L2p = (numpy.sqrt(PosiVect[0]**2 + PosiVect[1]**2 + PosiVect[2]**2))/1000.0 # [m]
            energy = Ei - 0.5 * mn * (L2p*L2p)/(tof*1.0e-6 - Lms/vi)**2 * J2meV
                #### removed and added by inamura (2008.09.04)

            ectemp = ecm.PutPointer(ind0).PutPointer(ind1)
            ectemp.Add("energy", l2d(energy), "meV")
            ectemp.SetKeys("energy", ykey, ekey)
            ectemp.Remove(xkey)
    hd = ecm.PutHeaderPointer()
    strvector = hd.PutStringVector("DATAPROCESSED")
    strvector.append("TOF to ENERGY CONVERSION")
    hd.OverWrite("DATAPROCESSED", strvector)
    return ecm

#########################################
def Tof2EnergyWhite(ecm, t_offset):        #Added by inamura (2008.11.20)
    """
    Function to convert Time-of-Flight to Energy on White beam
    
    Arguements:
        ecm : Elememt Container Matrix
        t_offset (microsec) : time-of-flight shift. 
    """
    import numpy
    if ecm(0).PutHeaderPointer().PutString("TYPE")=="MONITOR":
        start_ind0 = 1
    else:
        start_ind0 = 0
    L1 = ecm.PutHeaderPointer().PutDouble("L1") # [mm]
    L1 = L1/1000.0 #[m]
    for ind0 in range(start_ind0,ecm.PutTableSize()):
        print "psd = ",ind0
        for ind1 in range(ecm(ind0).PutTableSize()):
            xkey = ecm(ind0, ind1).PutXKey()
            ykey = ecm(ind0, ind1).PutYKey()
            ekey = ecm(ind0, ind1).PutEKey()
            tof_o = ecm(ind0,ind1).PutX()
            for i in range(len(tof_o)):
                if tof_o[i]==0:
                    tof_o[i]=1.0e-6
            tof = numpy.array(tof_o) - t_offset
            mn = 1.6749e-27  # Neutron mass
            J2meV = 6.24151e21 # Joule to meV
            
            PosiVect = ecm(ind0, ind1).PutHeaderPointer().PutDoubleVector("PixelPosition")
            L2p = (numpy.sqrt(PosiVect[0]**2 + PosiVect[1]**2 + PosiVect[2]**2))/1000.0 # [m]
            energy = 0.5 * mn * ((L1+L2p)*(L1+L2p))/(tof*1.0e-6)**2 * J2meV
            
            ectemp = ecm.PutPointer(ind0).PutPointer(ind1)
            ectemp.Add("energy", l2d(energy), "meV")
            ectemp.SetKeys("energy", ykey, ekey)
            ectemp.Remove(xkey)
    hd = ecm.PutHeaderPointer()
    strvector = hd.PutStringVector("DATAPROCESSED")
    strvector.append("TOF to ENERGY CONVERSION")
    hd.OverWrite("DATAPROCESSED", strvector)
    return ecm


#########################################
def NormalizedByMonitor(ecm_sample, monitorindex = 0, ini = None, fin = None): 
                        # changed by inamrua (2008.09.08)
    """
    Normalization by Monitor
    
    @param  dat             Params 
    @param  monitorindex    int     index of monitor for normalization
    @param  ini             float   inital value for integrating monitor data
    @param  fin             float   final value for integrating monitor data
    @retval ECM             elementContainerMatrix
    """
    t_i = ini     # added by inamura(2008.09.03)
    t_f = fin     # added by inamura(2008.09.03)
    monitorhistogram = simplehistogram(ecm_sample(0, monitorindex)) # changed by inamura (2008.09.03)
    m_i, m_e = monitorhistogram.sum(t_i, t_f)
    for ecaindex in range(1, ecm_sample.PutTableSize()):
        for ecindex in range(ecm_sample(ecaindex).PutTableSize()):
            '''
            ectemp = ecm_sample.PutPointer(ecaindex).PutPointer(ecindex)
            xk, yk, ek = ectemp.PutXKey(), ectemp.PutYKey(), ectemp.PutEKey()
            y0 = numpy.array(ectemp.PutY(), dtype='float')
            e0 = numpy.array(ectemp.PutE(), dtype='float')
            y1 = y0/m_i
            e1 = numpy.sqrt((e0/m_i)**2 + (m_e*y0/m_i**2)**2)
            newy = l2d(y1)
            newe = l2d(e1)
            ectemp.Replace(yk, newy)
            ectemp.Replace(ek, newe)
            ectemp.SetKeys(xk, yk, ek)
            '''
            ecm_sample(ecaindex,ecindex).MulMySelf(1/m_i) # changed by inamura (2008.09.03)
    hd = ecm_sample.PutHeaderPointer()
    strvector = hd.PutStringVector("DATAPROCESSED")
    strvector.append("MonitorCountNormalization")
    hd.OverWrite("DATAPROCESSED", strvector)
    return ecm_sample
    
    
    
    
def Rebin(ecm, Ei, Ef, dE):
    earray0 = numpy.linspace(Ei, Ef, dE)
    earray = l2d(earray0)
    for ind0 in range(ecm.PutTableSize()):
        for ind1 in range(ecm(ind0).PutTableSize()):
            tempec=ecm.PutPointer(ind0).PutPointer(ind1)
            tempec.Rebin(earray)
    return ecm
        



from __future__ import print_function
import Manyo.Utsusemi as mu


def UtsusemiVersion(isConsole=True):
    utsusemi_manyo_version = mu.UTSUSEMIMANYOVERSION
    try:
        utsusemi_manyo_version_rev = mu.UTSUSEMIMANYOVERSIONREV
    except:
        utsusemi_manyo_version_rev = 0

    ss = "---------------------------------------------\n"
    ss += "               Utsusemi Info\n"
    ss += "---------------------------------------------\n"
    if utsusemi_manyo_version_rev != 0:
        ss += "         version :  %s (rev %d)\n" % (
            utsusemi_manyo_version, utsusemi_manyo_version_rev)
    else:
        ss += "         version : %s \n" % (utsusemi_manyo_version)
    ss += "  Contact person : %s\n" % (mu.UTSUSEMICONTACTPERSON)
    ss += "          e-mail : %s\n" % (mu.UTSUSEMICONTACTEMAIL)
    ss += "---------------------------------------------\n"
    if isConsole:
        print(ss)
    else:
        return ss


UtsusemiVersion()



#!/usr/bin/python
import Manyo as m
API = m.AcquireNeutronSourceTextInformation()
##################################################
def valInfo4(sig,begin,end,step):
    valDate=[]
    valTime=[]
    valData=[]
    print "valInfo-4 >>>", sig, begin, end, step

    valData = API.PutValueInformation(sig,begin,end,step)
    valDate = API.PutDateBinInformation()
    valTime = API.PutTimeBinInformation()

    return valDate,valTime,valData
##################################################
def valInfo2(sig,begin):
    #valDate=[]
    valTime=[]
    valData=[]
    print "valInfo-2 >>>", sig, begin

    valData = API.PutValueInformation(sig,begin)
    valDate = API.PutDateBinInformation()
    valTime = API.PutTimeBinInformation()

    retDate = valDate[0]
    retTime = valTime[0]

    return retDate,retTime,valData
##################################################
def valInfo1(sig):
    #valDate=[]
    valTime=[]
    valData=[]
    print "valInfo-1 >>>", sig

    valData = API.PutValueInformation(sig)
    valDate = API.PutDateBinInformation()
    valTime = API.PutTimeBinInformation()

    retDate = valDate[0]
    retTime = valTime[0]

    return retDate,retTime,valData
##################################################
def toCurrent(Proton,bin):
    current=[]
    print "toCurrent >>>", len(Proton),bin

    current = API.ConvertProtonToCurrent(Proton,bin)

    return current
##################################################
def toBeampower(Current):
    beampower=[]
    print "toBeamPower >>>", len(Current)

    beampower = API.ConvertCurrentToBeampower(Current)

    return beampower
##################################################

listDate, listTime, listData = valInfo4("H2in","2009/06/03 10:00:00","2009/06/03 10:01:00",1)
#listDate, listTime, listData = valInfo4("H2cm","2009/06/03 10:00:00","2009/06/03 10:01:00",1)
#listDate, listTime, listData = valInfo4("H2dm","2009/06/03 10:00:00","2009/06/03 10:01:00",1)
#listDate, listTime, listData = valInfo4("H2pm","2009/06/03 10:00:00","2009/06/03 10:01:00",1)

#listDate, listTime, listData = valInfo4("CT8","2009/06/03 10:00:00","2009/06/03 10:00:30",1)
#listDate, listTime, listData = valInfo4("CT9","2009/06/03 10:00:00","2009/06/03 10:00:30",1)

print
print "      ##### result #####"
for i in range(len(listData)):
    print "[",i,"] date, time, val = ", listDate[i], listTime[i], listData[i]
print

#listDate, listTime, listData = valInfo2("H2in","2009/06/01 00:00:00")
#listDate, listTime, listData = valInfo1("H2in")
#listDate, listTime, listData = valInfo2("H2cm","2009/06/01 00:00:00")
#listDate, listTime, listData = valInfo1("H2cm")
#listDate, listTime, listData = valInfo2("H2dm","2009/06/01 00:00:00")
#listDate, listTime, listData = valInfo1("H2dm")
#listDate, listTime, listData = valInfo2("H2pm","2009/06/01 00:00:00")
#listDate, listTime, listData = valInfo1("H2pm")

#listDate, listTime, listData = valInfo2("CT8","2009/06/01 00:00:00")
#listDate, listTime, listData = valInfo1("CT8")
#listDate, listTime, listData = valInfo2("CT9","2009/06/01 00:00:00")
#listDate, listTime, listData = valInfo1("CT9")

#print
#print "      ##### result #####"
#print "date, time, val = ", listDate, listTime, listData
#print


#listDate, listTime, listProton = valInfo4("CT8","2009/06/03 10:00:00","2009/06/03 10:00:10",2)
#listCurrent = toCurrent(listProton,2)
#listBeampower = toBeampower(listCurrent)
#print
#print "      ##### result #####"
#for i in range(len(listProton)):
#    print "[",i,"]date,time,Proton,Current,Beam =", listDate[i], listTime[i], listProton[i], listCurrent[i], listBeampower[i]
#print



# -*- coding: utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        
# Purpose:     
#              
# Author:      
#
# Created:     27/01/2015
# Copyright:   (c)
# Licence:
#-------------------------------------------------------------------------------

import sys
from pyqtgraph.Qt import QtCore, QtGui, uic
import pyqtgraph as pg
import pyqtgraph.opengl as gl
import numpy as np
from OpenGL.GL import *

class GLAxisItem2(gl.GLAxisItem):
    def __init__(self):
        gl.GLAxisItem.__init__(self)

        self.lw=1.0
        self.lt=0
        self.xColor=[1, 0, 0, .6]
        self.yColor=[1, 1, 0, .6]
        self.zColor=[0, 1, 0, .6]
        self.origin=[0.0, 0.0, 0.0]
        pass

    def setOrigin(self, ori):
        self.origin = ori

    def setLineColor(self,x,y,z):
        self.xColor=x
        self.yColor=y
        self.zColor=z

    def getLineColor(self):
        return [self.xColor,self.yColor,self.zColor]

    def setLineWidth(self,v):
        self.lw=v

    def getLineWidth(self):
        return self.lw

    def setLineType(self,v):
        self.lt=v

    def getLineType(self):
        return self.lt

    def paint(self):
        self.setupGLState()

        if self.antialias:
            glEnable(GL_LINE_SMOOTH)
            glHint(GL_LINE_SMOOTH_HINT, GL_NICEST)

        glLineWidth(self.lw)
        if self.lt==0:
            glBegin( GL_LINES )
        else:
            if self.lt==1:
                glLineStipple(1, 0xAAAA)
            elif self.lt==2:
                glLineStipple(2, 0xAAAA)
            elif self.lt==3:
                glLineStipple(3, 0xAAAA)
            elif self.lt==4:
                glLineStipple(4, 0xAAAA)
            elif self.lt==5:
                glLineStipple(1, 0x00FF)
            elif self.lt==6:
                glLineStipple(2, 0x00FF)
            elif self.lt==7:
                glLineStipple(1, 0x0C0F)
            else:
                glLineStipple(3, 0x0C0F)
            glEnable(GL_LINE_STIPPLE);
            glBegin(GL_LINE_STRIP);

        x,y,z = self.size()
        x0 = self.origin[0]
        y0 = self.origin[1]
        z0 = self.origin[2]

        glColor4f(self.zColor[0],
                  self.zColor[1],
                  self.zColor[2],
                  self.zColor[3])
        glVertex3f(x0, y0, z0)
        glVertex3f(x0, y0, z0+z)

        glColor4f(self.yColor[0],
                  self.yColor[1],
                  self.yColor[2],
                  self.yColor[3])
        glVertex3f(x0, y0, z0)
        glVertex3f(x0, y0+y, z0)

        glColor4f(self.xColor[0],
                  self.xColor[1],
                  self.xColor[2],
                  self.xColor[3])
        glVertex3f(x0, y0, z0)
        glVertex3f(x0+x, y0, z0)
        glEnd()

        if self.lt==0:
            pass
        else:
            glDisable(GL_LINE_STIPPLE);
